***********************************************************************************************
*GENERATE CORRELATIONS BETWEEN CROSS SECTIONAL PROFITS AND AGGREGATE STOCK VOLATILITY
*PRODUCED TABLE1, COLUMNS 1, 2 AND 3
***********************************************************************************************

clear
set more off
set memory 500m

**************
***STAGE 1**** 
**************
**THIS DATA COMES FROM WRDS AND CAN BE SENT ON REQUEST, BUT IS TOO LARGE TO HOST ON MY WEBSITE (IT IS 150MB)
/*Insheeting raw Compustat quarterly data
insheet using quarterly_2007a.csv,names comma
gen sales=real(data2)
gen pretax=real(data23)
sort gvkey
cap drop qtr
ren spccqtr qtr
gen yq=year+(qtr-1)/4
ren dnum sic4
keep sales pretax gvkey yq sic qtr
*Drop if missing if incredible numbers
drop if sales==.|sales==0|pretax==.
*Drop last three quarters figures as these are estimated before accounts are published
egen lyear=max(yq),by(gvkey)
drop if yq-lyear>-0.51
*Only keep firms with 25 years of data to reduce entry/exit effects
egen noj=count(pretax),by(gv)
drop if noj<150
drop lyear noj
sa quarterly,replace
*/

**************
***STAGE 2****
**************
u quarterly,replace
* Growth rate of pretax profit
so gv yq
by gv: gen dpretax=(pretax-pretax[_n-1])/(0.5*(sales+sales[_n-1]))
*Windsorize extreme values
egen plow=pctile(dpretax),p(.05)
egen phigh=pctile(dpretax),p(99.95)
replace dpretax=phigh if dpretax>phigh&dpretax~=.
replace dpretax=plow if dpretax<plow

**************
***STAGE 3****
**************
*u nick,clear
*Generate cross-sectional profit growth numbers within quarter
egen sddpretax=sd(dpretax),by(yq)
*Generate cross-sectional profit growth numbers within SIC3 within quarter
gen sic3=floor(sic4/10)
egen _sicsddpretax=sd(dpretax),by(yq sic3)
*Take average across SIC3 for each quarter
egen sicsd=mean(_sicsddpretax),by(yq)
drop _sic
so yq
drop if yq==yq[_n-1]
sa profits,replace

**************
***STAGE 4****
**************
*Now insheet the stock-market volatility data
clear
insheet using vardata.csv
keep year month volatbl
gen quarter=floor((month-1)/3)
gen yq=year+quarter/4
*Going down to quartery to match to the quarterly profits data
collapse volatbl quarter year,by(yq)
so yq
merge yq using profits

**************
***STAGE 5****
**************
*Normalizing to 1 SD units and then run regressions
egen _volatbl=sd(volatbl)
gen xvolatbl=volatbl/_volatbl
egen _sdpr=sd(sd)
gen xsdpr=sd/_sdpr
egen _sicsd=sd(sicsd)
gen xsicsd=sicsd/_sicsd
tab qtr,gen(qq)

*Table 1, column1
reg xvolatbl xsd 
*Table 1, column2
reg xvolatbl xsd yq qq*
*Table 1, column3
reg xvolatbl xsicsd yq qq*

