***********************************************************************************************
*GENERATE CORRELATIONS BETWEEN CROSS SECTIONAL STOCK RETURNS SPREADS AND AGGREGATE STOCK VOLATILITY
*PRODUCED TABLE1, COLUMNS 4, 5 AND 6
***********************************************************************************************

clear
set more off
set memory 500m

**************
***STAGE 1****
**************
**THIS DATA COMES FROM WRDS AND CAN BE SENT ON REQUEST, BUT IS TOO LARGE TO HOST ON MY WEBSITE (IT IS 100MB)
/*Insheet the raw monthly stock returns data - a huge job
insheet using monthlystock.csv,names comma
gen r=real(ret)
drop if r==.
drop ret 
gen y=int(date/10000)
replace date=date-y*10000
gen m=int(date/100)
drop date
compress
egen noj=count(r),by(permno)
drop if noj<500
drop noj
sa monthly,replace
*/

**************
***STAGE 2****
**************
*Merge the monthly stock returns data with firm-level industry codes
u monthly,clear
gen ym=y+(m-1)/12
so permno
merge permno using permnosic
drop if _m==2
drop _m
gen sic3=int(siccd/10)
egen sic=count(r),by(ym sic3)
*sa nicktemp,replace

**************
***STAGE 3****
**************
*Generate cross sectional spread measures, first trim some extreme outliers
*u nicktemp,clear
egen cc=count(r),by(ym)
*Windsorize extreme values
egen pl=pctile(r),p(.5)
egen ph=pctile(r),p(99.5)
replace r=pl if r<pl
replace r=ph if r>ph&r~=.
drop ph pl
*Generate plain cross sectional standard-deviation for each month
egen sdr=sd(r),by(ym)
*Generate this for each month within SIC3
egen _sicsdr=sd(r),by(ym sic3)
*Obtain the average monthly within SIC3 spread - note do not trim of #obs per sic, to keep sample as constant as possible over the panel
egen sicsd=mean(_sicsdr),by(ym)
drop _sicsdr
so ym
save monthlystock_all,replace
drop if ym==ym[_n-1]
save monthlystock,replace

**************
***STAGE 4****
**************
*Now insheet the stock-market volatility data
clear
insheet using vardata.csv
gen ym=year+(month-1)/12
so ym
merge ym using monthlystock
keep if sd~=.

**************
***STAGE 4****
**************
*Normalize so both measures had a SD=1, then run the Table 1 regressions
egen _volatbl=sd(volatbl)
gen xvolatbl=volatbl/_volatbl
egen _sd=sd(sd)
gen xsd=sd/_sd
egen _sicsd=sd(sicsd)
gen xsicsd=sicsd/_sicsd
qui tab month,gen(mm)
so ym

*Some d-stats showing that the cross-sectional spread is correlated over time with stock-market volatility
gen xsd1=xsd[_n+1]
gen xsd2=xsd[_n+2]
gen xsd3=xsd[_n+3]
cor xvolatbl xsd*

*Table 1: Columns 4
reg xvolatbl xsd 
*Table 1: Columns 5
reg xvolatbl xsd ym mm*
*Table 1: Columns 6
reg xvolatbl xsicsd ym mm*
predict x if e(sample)

*Number of units
su cc if x~=.