clear
set memory 200m
set matsize 3000

*Data from Livingstone survey on nominal gdp (tab GDPX) from http://www.phil.frb.org/files/liv/datai.html 

insheet using rawdata_oldGDP.csv
save oldgdp,replace
insheet using rawdata_newgdp.csv,clear
append using oldgdp

*Generate year+half time indicator
gen yh=real(substr(date,5,2))
replace yh=yh+1900 if yh>10
replace yh=yh+2000 if yh<10
replace yh=yh+0.5 if substr(date,1,3)=="Dec"
so yh

*Generate coefficient of variation on forecasts
gen m12=real(forecast12month)
drop forecast*
egen m=mean(m12),by(yh)
egen sd=sd(m12),by(yh)
replace sd=sd/m

*Only keep half-years with at least 50 forecasters
egen ynoj=count(sd),by(yh)
keep if ynoj>=50

so yh
drop if yh==yh[_n-1]
keep yh sd* ynoj
sa livingstonegdp,replace

*Now insheet the stock-market volatility data
clear
insheet using vardata.csv
keep year month volatbl
gen half=(month>6)
gen yh=year+half/2

*Going down to half-yearly to match to the Livingstone survey
collapse volatbl half year,by(yh)
so yh
merge yh using livingstonegdp

*Detrend livingstone
regress sd yh
predict res,res
*Then sd normalize both series
egen sdres=sd(res)
gen xliv=res/sdres
egen sdvolatbl=sd(volatbl)
gen xvolatbl=volatbl/sdvolatbl

*Generate some half-year dummies
tab half,gen(hh)
reg xvolatbl xliv
reg xvolatbl xliv half yh
