%                   NICK BLOOM (STANFORD) UNCERTAINTY SHOCK SIMULATION (VALUE FUNCTION ITERATION AND NUMERICAL SIMULATION)
%                   THIS VERSION AUGUST 2007.
%                   IF YOU HAVE ANY QUESTIONS OR FIND ANY BUGS PLEASE CONTACT ME AT NBLOOM@STANFORD.EDU, 650-725-7836, THANKS NICK    
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Usuage Notes:
% 1) This file is best viewed in Matlab onscreen as the color coding makes it much easier to view and the lines are not "wrapped"
% 2) The program normally runs on Matlab 7.0 for Linux in 64bit using 8GB of memory. If you get an "out of memory" command try reducing
%    the number of points in the state space by lowering the value of "gnum" and reducing the number macro loops ("macro") or plants ("nop")
%    in the simulation.
% 3) This file was created and run in Linux Matlab 7.4. R2007a Some commands may need to be changed for older versions (note my experience is
%    that Linux is worth the pain (in terms of compatability etc..) because it is faster and seems to never crash....
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   STAGE 1 - DEFINING THE PARAMETERS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%Next two lines commented out for interactive (simulation) mode
%function criterion=value(inputa,inputb,inputc,inputd,inpute,inputf,inputg,inputh,inputj,inputk)
%global v2 runs g0 h0 l0 tranvarm swopcc swopvar returns_init cc0full g0_long h0_long extra muist must varst varist ccst ccist gst gnst lst lnst cap_now1 lab_now1 yyy_now inve emp varnum ccnum factor g0_detail l0_detail sizeg0_long gldetail indexst firms i0 e0 hours0k hhours vcv1 vcv labfull i0nonzero e0nonzero i0squaredcc e0squared i0negativecc e0negative i0cc simcoeff datacoeff vcv4 h0full v3 criterionfull vcvfull  gup gdown hup hdown istar estar mu_mt sales_mt yyy_mt cap_mt lab_mt dtfp_t dtfp_w dtfp_b dtfp_c cor_t ri_mt var_mt tot years ergodic l g inv_ inv_2 inv_4 emp_ emp_2 emp_4 dsales_ dsales_2 dsales_4 estargl istargl simulation nop gnum hnum column markup enum egap cap lab;
warning off all; 
%%%Next two lines commented out for estimation mode
 clear;  clear global; params=[0.0146    0.0208         0         0    0.3392    0.0181    0.0605         0    0.4432    2.0093]; column=1; %baseline
%clear;  clear global; params=[0.0208    0.0028    0.4612    0.3598    0.2362    0.0103    0.0687         0    0.4999    2.1290]; column=1; enum=2; %Columnr=2
%clear;  clear global; params=[0.0132         0    2.0562    0.0699    0.4698         0    0.0632         0    0.2481    2.1077]; column=1; %Columnr=4
%clear;  clear global; params=[0.0090    0.0141    0.5249         0    0.3525         0    0.0758         0    0.5154    2.0000]; column=1; %Capital ME
%clear;  clear global; params=[0.0161    0.0129    1.4884    0.8080    0.5000         0         0    0.2500    0.6000    2.1292]; column=1; %Weekly
%clear;  clear global; params=[0.0157    0.0160    0.0580         0    0.3613         0    0.0661         0    0.4699    2.0561]; column=1; %Initial conditions plus five percent

inputa=params(1);inputb=params(2);inputc=params(3);inputd=params(4); inpute=params(5); inputf=params(6); inputg=params(7); inputh=params(8); inputj=params(9); inputk=params(10); runs=1; simulation=1; markup=[]; enum=[]; %0 for estimation set-up and 1 for simulation set-up


%NICK BLOOM, DECEMEBR 2003, VALUE FUNCATION-mean(emp_2)))^(1/2)),...
%A) Setting parameter values
cap=0.25; lab=0.5; if (size(markup))>0; cap=(1/3)*(1/1.2); lab=(2/3)*(1/1.2); else; markup=0; end; %cesparam=0.5;
fck=inputa; fcl=inputb; lambdak=inputc;lambdal=inputd; sellloss=inpute; hirefire=inputf; mud=inputg; mup=inputh; varscale=inputj; wagelam=inputk;
wage1=(wagelam-1)/wagelam; wage2=1/(1+(wagelam-1)*40^wagelam); %Pick parameter based on getting paid time & 1/2 for one extra shift. This implies a ratio of 1.08333 (6.5/6); (wage1.*(1./48+wage2.*48.^(wagelam-1)))/((wage1.*(1./40+wage2.*40.^(wagelam-1))));
scaling=1/3; %Meaningless scale parameter set so that values for main variables are not too small, as the computer precision starts to matter
r=0.065; dep=0.1; mu=0.02; 
period=12;  
GE=0; GEPRICE=0.5/100;GEWAGE=0.3/100; GEINT=1.1;%Include GE size price changes
hire=12*0.5*hirefire; fire=hire; %assume hiring/firing margin equally split between hiring and firing, and in terms of years wages (noting average weekly wages are 1)
                
gnum=100; %Ideally have about 500 points here but computer speed limits this to 100. Estimation results seem to be insensitive to local variations around 100 (i.e. compare 120 to 80)
%if simulation==1; scale=1; gstart=reallog(0.0033); gfinish=reallog(2); ginc=(gfinish-gstart)/(gnum-1); hstart=log(5*(exp(gstart-gfinish))^(1/scale)); if sum(params(1:6))==0; gstart=log(0.1); gfinish=log(2.5); ginc=(gfinish-gstart)/(gnum-1); hstart=log(1*(exp(gstart-gfinish))^(1/scale)); end; end %Cobb-Douglas, Higher lab number shifts to the right
if simulation==1; scale=1; gstart=reallog(0.0033); gfinish=reallog(2); ginc=(gfinish-gstart)/(gnum-1); hstart=log(5*(exp(gstart-gfinish))^(1/scale)); if column==4; scale=2; gnum=140; gstart=reallog(0.02); gfinish=reallog(0.5);  ginc=(gfinish-gstart)/(gnum-1); hstart=log(1*(exp(gstart-gfinish))^(1/scale)); end; end %Cobb-Douglas, Higher lab number shifts to the right
%if simulation==0; scale=1.33; gstart=reallog(0.00033); gfinish=reallog(5); ginc=(gfinish-gstart)/(gnum-1); hstart=log(6*(exp(gstart-gfinish))^(1/scale)); end %Cobb-Douglas, Higher lab number shifts to the right
if simulation==0; scale=1.5; gstart=reallog(0.001); gfinish=reallog(2.5); if markup==1; gstart=reallog(0.0004); gfinish=reallog(1.5); end; ginc=(gfinish-gstart)/(gnum-1); hstart=log(6*(exp(gstart-gfinish))^(1/scale)); if column==4; scale=2; gnum=140; gstart=reallog(0.02); gfinish=reallog(0.5);  ginc=(gfinish-gstart)/(gnum-1); hstart=log(1*(exp(gstart-gfinish))^(1/scale)); end; end %Cobb-Douglas, Higher lab number shifts to the right
if max(size(enum))==0; enum=0; end; egap=(1-exp(enum*-ginc)); %How much extra labor depreciates over capital each period
%gnum=30; scale=0.5;   gstart=reallog(0.5); gfinish=reallog(1.25); ginc=(gfinish-gstart)/(gnum-1); %Labor only
hnum=round(gnum/scale);minhour=25;maxhour=55; %Cap hours at extreme values
hinc=ginc; hfinish=(hnum-1)*hinc+hstart; dept=(1+dep)^(1/period)-1;
var0=(inputj*[1,2]).^2; varnum=2; swopvar=[1-1/36 1/36;0.29 0.71];
mu0=[mu-mud,mu+mud]; munum=2; swopmu=[1-mup mup; mup 1-mup];

%Setting up the shock transition matrix called tranvarm which is indexed by (gnow,gnext,sigma,mu)
     extra=2*round(gnum/4);
     g0=exp(gstart:ginc:gfinish);h0=exp(hstart:hinc:hfinish);
     g0_long=[exp(gstart-(extra/2)*ginc:ginc:gstart-ginc),g0,exp(gfinish+ginc:ginc:gfinish+(extra/2)*ginc)]; sizeg0_long=size(g0_long,2);
     h0_long=[exp(hstart-(extra/2)*hinc:hinc:hstart-hinc),h0,exp(hfinish+hinc:hinc:hfinish+(extra/2)*hinc)];
     i0=single(((repmat(permute(repmat(g0,[gnum 1])./repmat(g0',[1 gnum]),[1 3 2 4]),[1 hnum 1 hnum])-1)));
     e0=single(((repmat(permute(repmat(h0',[1 hnum])./repmat(h0,[hnum 1]),[3 1 4 2]),[gnum 1 gnum 1])./permute(repmat(repmat(g0',[1 gnum])./repmat(g0,[gnum 1]),[1 1 hnum hnum]),[1 3 2 4])-1 )))+egap;
     factor=repmat(single(((1-dept)/((1+r.*permute(repmat([1 (1-(GE*GEINT/100)/r)]',[1 gnum hnum gnum hnum]),[ 2 3 4 5 1])).^(1/period))).*repmat(1+i0,[1 1 1 1 varnum])),[1 1 1 1 1 munum]); %GE
     gldetail=4; glextra=2*round(gnum/10); gincd=ginc/gldetail; hincd=hinc/gldetail;l0=1./h0; %lnum=hnum; 
     g0_detail=[exp(gstart-(glextra/2)*gincd:gincd:gstart-gincd),exp(gstart:gincd:gfinish),exp(gfinish+gincd:gincd:gfinish+(glextra/2)*gincd)];
     l0_detail=1./[exp(hstart-(glextra/2)*hincd:hincd:hstart-hincd),exp(hstart:hincd:hfinish),exp(hfinish+hincd:hincd:hfinish+(glextra/2)*hincd)]; %Note l0 is the inverse of h0, comes in useful later as employment liner in l0 but not h0
     i0nonzero=round(1.e+3*i0)~=0; e0nonzero=round(1.e+3*(e0))~=0; 
%end;
     hours0=single(max(min((scaling*lab/(wage1*wage2*wagelam)).^(1/(wagelam-lab)).*((repmat(permute(shiftdim(g0.^((1-cap-lab)/(wagelam-lab)),-2),[1 2 4 3]),[gnum hnum 1 hnum]))).*((repmat(permute(shiftdim(h0.^((lab-1)/(wagelam-lab)),-2),[1 2 3 4]),[gnum hnum gnum 1]))),maxhour),minhour));    
     hours0k=hours0.^wagelam; hhours=permute(hours0(1:varnum,1:munum,:,:,1),[3 4 1 2]);
     %RETURNS per period (month). Dimensions are (investment, employment, capital, labour, uncertainty, drift)
     returns_init =single((scaling/period)*((single(permute(repmat(g0',[1 hnum gnum hnum]),[3 2 1 4])))).^(1-cap-lab).*(((permute(repmat(h0,[gnum 1 gnum hnum]),[1 4 3 2]))).*hours0).^lab);  %Cobb-Douglas
     %returns_init =single((1/period)*(((single(permute(repmat(g0',[1 hnum gnum hnum]),[3 2 1 4])))).^cesparam + (((permute(repmat(h0,[gnum 1 gnum hnum]),[1 4 3 2]))).*hours0).^cesparam)).^((cap+lab)/cesparam);  %CES

for s=1:varnum; for i=1:gnum; for m=1:munum;
     %setting up the values for within period mean and variance - but first correct for linear to log change, using value at sample average
     %gs=g0(gnum/2);loglinear=((normcdf(g0.*(1+ginc/2),gs,gs*(var0(1)/period).^(1/2))-normcdf(g0.*(1-ginc/2),gs,gs*(var0(1)/period).^(1/2)))*log(g0') - (normcdf(g0.*(1+ginc/2),gs,gs*(var0(2)/period).^(1/2))-normcdf(g0.*(1-ginc/2),gs,gs*(var0(2)/period).^(1/2)))*log(g0'))/((var0(2)/period)-(var0(1)/period)); %Simulation estimate of var correction        
     mugt=(1+mu0(m)+dep-0.5*var0(s))^(1/period)-1; sdgt=((var0(s))/period)^(1/2);  %Corrected value with -var/2 log corection (so that in levels drift is mu). Robust to dropping this as 2nd order effects
     tranvarm(i,:,s,m)=normcdf(log(g0_long)+ginc/2,mugt+log(g0_long(1))+(i+extra/2-1)*ginc,sdgt)-normcdf(log(g0_long)-ginc/2,log(g0_long(1))+mugt+(i+extra/2-1)*ginc,sdgt);
end; end; end; tranvarm=single(tranvarm);
%Generate monster transition matrix for the one-step expectations calculation - dimension (gnum,hnum,varnum,munum,gnum+extra,varnum,munum), structure (gnow,.,varnow,munow,gnext,varnext,munext)
monster_tran=repmat(permute(tranvarm,[1 5 3 4 2 6 7]),[1 hnum 1 1 1 varnum munum]).*repmat(permute(swopvar,[3 5 1 6 4 2 7]),[gnum hnum 1 munum gnum+extra 1 munum]).*repmat(permute(swopmu,[3 5 6 1 4 7 2]),[gnum hnum varnum 1 gnum+extra varnum 1]);

%Dimension (gnext,hnext,gnow,hnow). G is A/K and H is L/K
returns=single(repmat(returns_init,[1 1 1 1 varnum]).*permute(repmat([1 1-GE*(GEPRICE)]',[1 gnum hnum gnum hnum]),[2 3 4 5 1]).*(1-(period*fck)*repmat(i0nonzero ,[1 1 1 1 varnum]) - (period*fcl)*repmat(e0nonzero,[1 1 1 1 varnum]))-   (((repmat(permute(h0/period,[3 4 1 2 5]),[gnum hnum gnum 1 varnum]))).*wage1.*(1+wage2.*repmat(hours0k,[1 1 1 1 varnum])).*(permute(repmat([1 1-GE*(GEWAGE)]',[1 gnum hnum gnum hnum]),[2 3 4 5 1])) + repmat(permute(h0,[3 4 1 2 5]),[gnum hnum gnum 1 varnum]).*(repmat(e0,[1 1 1 1 varnum]).*(hire -(fire+hire).*(repmat(e0,[1 1 1 1 varnum])<0)) + lambdal*(period/12)*repmat(e0,[1 1 1 1 varnum]).^2))      -  (repmat(i0,[1 1 1 1 varnum]).*(1 - sellloss.*(repmat(i0,[1 1 1 1 varnum])<0)).*permute(repmat([1 1-GE*(GEPRICE)]',[1 gnum hnum gnum hnum]),[2 3 4 5 1]) + lambdak*(period/12).*repmat(i0,[1 1 1 1 varnum]).^2)); %GE
returns=repmat(returns,[1 1 1 1 1 munum]); %sizing up

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   STAGE 2 - RUNNING THE NUMERICAL CONTRACTION MAPPING (SOLVING THE VALUE MAXIMISATION ITERATION)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%STAGE 2 - RUNNING THE VALUE MAXIMISATION ITERATION
v1=single((1-lab)/cap*ones(gnum,hnum,varnum,munum)); %Use fresh value every time for consistency around iterations (i.e. do not use last periods value function)     
Total=150*(period/12); %note - policy correspondences converge very fast (basically completely by 150 loops) and much faster then value functions

for n=1:Total;
%Linear extrapolates using result that in adjustement zone value is linear with fixed and/or partial irreversibility, and approximately linear with moderate quadratic adjustment costs
for is=1:varnum; for ic=1:munum; for jh=1:hnum; v1_extrap(:,jh,is,ic)=[(v1(2,jh,is,ic)-v1(1,jh,is,ic)).*((g0_long(1:extra/2)-g0(1))/(g0(2)-g0(1))) + v1(1,jh,is,ic),v1(:,jh,is,ic)',(v1(end,jh,is,ic)-v1(end-1,jh,is,ic)).*((g0_long(gnum+extra/2+1:end)-g0(end))/(g0(end)-g0(end-1))) + v1(end,jh,is,ic)]; end; end; end;

%Multiply together monster transition and monster value function, first create monster value function
monster_value=repmat(permute(v1_extrap,[5 2 6 7 1 3 4]),[gnum 1 varnum munum 1 1 1]);
Ev1=dot(monster_tran(:,:,:,:,:),monster_value(:,:,:,:,:),5);

%The contraction mapping stage
if n<Total;
     %This single line accounts for about 60% of the run time. The order and selection of commands has been chosen  to optimise run speed.
     v1=squeeze(max(max(returns +  factor.*repmat(permute(Ev1,[1 2 5 6 3 4]),[1 1 gnum hnum 1 1]))));
else end;
if simulation==1; disp([n/1000,v1(10,10,2,2)]); end; %Can use this line to follow what is happening in simulation mode
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   STAGE 3:GENERATING OPTIMAL POLICY CORRESPONDENCE DATA (THE OPTIMAL INVESTMENT FUNCTIONS ETC.)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Solving for optimal investment and employment. Vectorize to avoid looping as much faster
[m1 i1]=max(returns +  factor.*repmat(permute(Ev1,[1 2 5 6 3 4]),[1 1 gnum hnum 1 1]));
%clear returns;
[m2 i2s]=max(squeeze(m1));
v2=squeeze(m2); clear m1 m2;
i2s=reshape(i2s,gnum*hnum*varnum*munum,1);
i1s=i1(i2s+[0:gnum*hnum*varnum*munum-1]'.*hnum); clear i1;
gi=single(reshape(repmat([1:gnum],[1 hnum varnum munum]), gnum*hnum*varnum*munum,1));
hi=single(reshape(repmat(ones(gnum,1)*[1:hnum],[1 1 varnum munum]), gnum*hnum*varnum*munum,1));
istar=single(reshape(g0(gi)./g0(i1s)-1,[gnum hnum varnum munum]));
estar=single(reshape((g0(gi)./g0(i1s)).*(h0(i2s)./h0(hi))-1,[gnum hnum varnum munum]));
gstar=single(reshape(g0(i1s),[gnum hnum varnum munum]));

%Edit this out when running the actual simulation as only used for looking at results
hstar=single(reshape(h0(i2s).*(g0(gi)./g0(i1s)),[gnum hnum varnum munum]));
hoursstar=min(max(single((scaling*lab/(wage1*wage2*wagelam)).^(1/(wagelam-lab)).*(repmat(g0',[1 hnum varnum munum])).^((1-cap-lab)/(wagelam-lab)).*((repmat(h0,[gnum 1 varnum munum])).^((lab-1)/(wagelam-lab)))),minhour),maxhour);
     
gup=[];gdown=[]; g0=single(g0); h0=single(h0);
for bh=1:hnum; for cs=1:varnum; for cc=1:munum;
m=max(g0'.*(g0'==gstar(:,bh,cs,cc))); gup(bh,cs,cc)=m; 
m=min(g0'.*(g0'==gstar(:,bh,cs,cc))+g0(gnum).*(1-(g0'==gstar(:,bh,cs,cc)))); gdown(bh,cs,cc)=m; 
end; end; end;
hup=[];hdown=[];for bh=1:gnum; for cs=1:varnum; for cc=1:munum;
m=max(h0.*(h0*(1-egap)==hstar(bh,:,cs,cc)));hup(bh,cs,cc)=m;
m=min(h0.*(h0*(1-egap)==hstar(bh,:,cs,cc))+h0(hnum).*(1-(h0*(1-egap)==hstar(bh,:,cs,cc)))); hdown(bh,cs,cc)=m;
end; end; end;

%Key plot - can run at this stage to work out what the results look like
%close;m1=1; m2=1;var1=1;var2=2; plot(log(gup(:,var1,m1)'./h0),log(gup(:,var1,m1)),'r',log(gdown(:,var1,m1)'./h0),log(gdown(:,var1,m1)),'r',log(g0'./hup(:,var1,m1)),log(g0),'r',log(g0'./hdown(:,var1,m1)),log(g0),'r',log(gup(:,var2,m2)'./h0),log(gup(:,var2,m2)),'b',log(gdown(:,var2,m2)'./h0),log(gdown(:,var2,m2)),'b',log(g0'./hup(:,var2,m2)),log(g0),'b',log(g0'./hdown(:,var2,m2)),log(g0),'b');title('Invest, Disinvest, Hire and Fire on (Y/K,Y/L) space - Low, Medium and High Uncertainty'); 

%NEED TO SPEED UP THIS EXTRAPOLATION - SHOULD REALLY VECTORIZE THIS PART BUT NOT THAT SLOW SO LEFT FOR PRESENT
gnumd=length(g0_detail); lnumd=length(l0_detail);
for s=1:varnum; for cc=1:munum; for h=1:hnum; 
                                             istarg(:,h,s,cc)=interp1(g0',istar(:,h,s,cc),g0_detail','linear','extrap');
                                             estarg(:,h,s,cc)=interp1(g0',estar(:,h,s,cc),g0_detail','linear','extrap');
                                             hoursg(:,h,s,cc)=interp1(g0',hhours(:,h,s,cc),g0_detail','linear','extrap');
                                             valueg(:,h,s,cc)=interp1(g0',v1(:,h,s,cc),g0_detail','linear','extrap'); 
                                             end;end;end;
                                 
for s=1:varnum; for cc=1:munum; for g=1:gnumd;
                                             istargl(g,:,s,cc)=interp1(l0',istarg(g,:,s,cc),l0_detail','linear','extrap');
                                             estargl(g,:,s,cc)=interp1(l0',estarg(g,:,s,cc),l0_detail','linear','extrap');
                                             hoursgl(g,:,s,cc)=interp1(l0',hoursg(g,:,s,cc),l0_detail','linear','extrap');
                                             valuegl(g,:,s,cc)=interp1(h0',valueg(g,:,s,cc),l0_detail','linear','extrap'); 
                                             end; end; end;
estargl=min(max(estargl,-0.8),10); istargl=min(max(istargl,-0.8),5); hoursgl=min(max(hoursgl,minhour),maxhour); %safety precaution against extrapolation on extreme values

%}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   STAGE 4: RUNNING THE DATA SIMULATIONS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Clear some unnecessary variables before start the simulation
clear ystarg ystargl lstarg lstargl gstarg gstargl istarg estarg ykstargl ylstargl ykstarg ylstarg v2_extrap valueg cashg temp tranvarm tranvarm_g returns index index1 index2 vv2 i0full e0full returns1 h0full g0full inve emp dy cap_t cap_now lab_t lab_now yyy_t yyy_now cap_tf dyhistory cc0full factor returns capexr_tf hiring_tf stdr_tf ihistory ehistory dyhistory var_tf vhistory yyy_y;  
clear cap_mt lab_mt sales_mt yyy_mt ri_mt var_mt mu_mt tfp_mt dtfp* monster_value hours capex* cap_* lab_* sales_* dy* hours1 hours2 yyy_* sd*

%Set some of the monthly parameters
dept=dep/period; ldept=(dep+egap)/period; egapt=egap/period; var0p=(var0/period).^(1/2); mu0p=mu0/period; %Note var0p is really the monthly sd of the shocks

%Some of the experiments (note these also need changes in the normalization in the plots at the end)
volatility=0;  %Fixes volatility to stay constant while only uncertainty changes
uncertainty=0; %Fixes uncertainty to stay constant while only volatility changes
firstmoment=0; %Allows a first moment shock as well

%Setting up number of firms, running time and replications for the estimation and simulation modes
if simulation==0
    macro=10; if max(size(nop))==0; nop=250; end; firms=1000; yearreg=20;
else; %Reduce number of firms here if problems with memory on simulations - try 1000 or even 250 for example
    macro=25000; nop=250; firms=1000/nop; yearreg=4; ssales=ones(1,macro*firms*nop); cap_nowf=ones(1,macro*firms); lab_nowf=cap_nowf; sales_nowf=cap_nowf; share=ones(1,macro*firms*nop); TFP_current=1;clear returns* factor* e0* i0* monster* hours0*;     
end;

ergodic=10; rand('state',0); randn('state',0); %Note need 4 lags so an ergodic of 7 provides 3 years before first data collected. Tradeoff in ergodic as larger values take longer and generate more cross-sectional size spread (less aggregation)
gincd=ginc/gldetail; lincd=-hinc/gldetail; 
months=period;
sharem=1/3.^(1/2); sharef=1/3.^(1/2); sharep=1/3.^(1/2); % Share macro,firm and plant of all total variance
years=ergodic+yearreg; 
for t=1:years*period;   %SIMULATION LOOP OVER PERIODS (MONTHS/WEEKS ETC.)
    year=floor(t/months);
  if t==1;    %FIRST PERIOD - THROW AWAY VALUES SO ONLY NEED TO SET UP AS RIGHT DIMENSIONS
%   if runs==1;  %Set up initial state for first period and use the same for all loops
   g=single(exp(log(gdown(round(hnum/2),1,1))+(log(gup(round(hnum/2)-1,1,1))-log(gdown(round(hnum/2),1,1))).*(rand(1,nop*firms*macro).^(1/6)))); if max(isnan(g))==1; g=(g<100); else if mean(g)==0; g=(g<100); end; end;;   %Good guess for initial distribution and then ergodic to run this in for 5 years
   l=single(1./hup(round(gnum/2),1,1)+((1./hdown(round(3*gnum/4),1,1))-(1./hup(round(gnum/2),1,1))).*rand(1,nop*firms*macro)); if max(isnan(l))==1; l=(l<100); else if mean(l)==0; l=(l<100); end; end;;
   ccap_now=ones(nop,firms*macro); cap_now=reshape(ccap_now,1,nop*firms*macro);cap_now1=cap_now; clear ccap_now; %Set up equal sizes to start, however
   lab_now=cap_now./l; lab_now1=lab_now; yyy_now=cap_now.*g;yyy_now1=yyy_now;
   %First setting up firm values for var and cc (both levels and indexes)
   varim=ones(1,macro)+(rand(1,macro)>( swopvar(2,1)/(swopvar(2,1)+swopvar(1,2)) ));  %Set up with ergodic steady-state
   varif=single(reshape(repmat(varim,[firms,1]),1,[]));
   varip=single(reshape(repmat(varim,[nop*firms,1]),1,[]));
   varm=var0p(ones(1,macro)); 
   varf=var0p(varif); 
   varp=var0p(varip); 
   muif=ones(1,firms*macro)+(rand(1,firms*macro)>0.5);  %Set up with ergodic steady-state
   muf=mu0p(muif); 
   muip =single(reshape(repmat(muif,[nop,1]),1,[]));
   %Then moving to plant values for var and cc (both levels and indexes)
   shockm=randn(1,macro); shockf=randn(1,firms*macro); shockp=randn(1,nop*firms*macro);
   dym=sharem*shockm.*varm; dym=reshape(repmat(dym,[nop*firms,1]),1,[]);
   dyf=muf + sharem*shockf.*varf; dyf=reshape(repmat(dyf,[nop,1]),1,[]);
   dyp=sharep*shockp.*varp;
   dy=max(min(dym+dyf+dyp,0.2),-0.2);
   g=g.*(1+dy); inve=single(zeros(1,firms*nop*macro)); emp=inve; hours=inve;

   else     %ALL SUBSEQUENT PERIODS
   %Note that: (i) dy varies by unit, firm and panel; (ii) var varies by panel; and (iii) mu varies by firm 
   varshock=single(rand(1,macro));
   varim1=varim; varim=(varim1==1).*(1+(varshock>swopvar(1,1)))+(varim==2).*(2-(varshock<swopvar(2,1)));
   if simulation==1; if  t==round(period*(ergodic)+1);  varim=2*(varim>0); end; end;    %Second moment shock
   varif1=varif; varif=single(reshape(repmat(varim,[firms,1]),1,[])); 
   varip1=varip; varip=single(reshape(repmat(varim,[nop*firms,1]),1,[]));
   varm=single(var0p(varim1));  %Note mean and variance depends on lagged values
   varf=single(var0p(varif1)); 
   varp=single(var0p(varip1)); 
   mushock=single(rand(1,firms*macro)); 
   muif1=muif; muif=single((muif1==1).*(1+(mushock>swopmu(1,1)))+(muif1==2).*(2-(mushock<swopmu(2,1))));
   muf=single(mu0p(muif1)); 
   muip1=muip; muip=single(reshape(repmat(muif,[nop,1]),1,[])); 
   shockm=single(randn(1,macro)); shockf=single(randn(1,firms*macro)); shockp=single(randn(1,nop*firms*macro));
   if volatility==1; varmcarry=varm; varm=(varm>0); varfcarry=varf; varf=(varf>0); varpcarry=varp; varp=(varp>0); end; %Imposes constant volatility of actual shocks (need to also change below)
   dym=sharem*shockm.*varm;
   dym=single(reshape(repmat(dym,[nop*firms,1]),1,[]));
   dyf=muf + sharem*shockf.*varf; dyf=single(reshape(repmat(dyf,[nop,1]),1,[]));
   dyp=sharep*shockp.*varp;
   if volatility==1; if simulation==1; varm=varmcarry; varf=varfcarry; varp=varpcarry; clear varmcaryy varfcarry varpcarry;  end; end; %Imposes constant volatility of actual shocks (need to also change above)
   dy=max(min(dym+dyf+dyp,0.2+mean(mu0p)),-0.2+mean(mu0p)); %Cap monthly shocks at 25% to stop process exploding and preserve aggregation effects for longer
   if simulation==1; if t==2; yyy_baseline=mean(yyy_now); else; dy=dy-mean(yyy_now.*dy)/mean(yyy_now); dy=dy+ (yyy_baseline.*(1+mean(mu0p))^(t+1) - mean(yyy_now.*(1+dy)+1.0e-10))./yyy_baseline;  if t==500; stop; end; end; end; %Autocorrects against trend for whole macro samples - stops random affects driving results (i.e. use sample of only 1000 macro)
   if firstmoment==1; simulation==1; if  t==round(period*(ergodic)+2);  dy=dy-0.01; yyy_baseline=yyy_baseline.*(1-0.01); end; end;    %First moment shock
   g=g.*(1+dy)./((1+inve).*(1-dept)); l=l.*(1+inve)./((1+emp).*(1-egapt));
   gn=min(max(1+round((log(g)-log(g0_detail(1)))/gincd),1),gnumd);
   ln=min(max(1+round((log(l)-log(l0_detail(1)))/lincd),1),lnumd);   %Note for editing, normalizing by gincd reverses the signage here
   index=(gn + (ln-1)*gnumd + (varip-1)*gnumd*lnumd + (muip-1)*gnumd*lnumd*varnum);
   if uncertainty==1; if simulation==1; index=(gn + (ln-1)*gnumd + (muip-1)*gnumd*lnumd*varnum); end; end;
   inve=istargl(index);
   emp=estargl(index); if simulation==1; hours1=hours;end
   hours=hoursgl(index);
   end;
     %%%Generating capital and yyy values    
     if simulation==1; end
     cap_now1=cap_now; lab_now1=lab_now; yyy_now1=yyy_now;
     cap_now=double(cap_now1.*(1+inve).*(1-dept)); lab_now=double(lab_now1.*(1+emp).*(1-ldept)); yyy_now=single(yyy_now1.*(1+dy)+1.0e-10); 
     if (t==months*floor(t/months)+1); year=floor(t/months); %Generating sales and capex values (note have to reset every 12 months and add over the year)
                 capexr=inve.*(1-(inve<0)*sellloss).*cap_now1;
                 sales=(yyy_now.*((cap_now1./yyy_now).^cap).*((lab_now1./yyy_now).*hours).^lab)./period; %Cobb-Douglas
     else 
                 capexr=capexr+inve.*(1-(inve<0)*sellloss).*cap_now1;
                 if simulation==0; sales =sales +(yyy_now.*((cap_now1./yyy_now).^cap).*((lab_now1./yyy_now).*hours).^lab)./period; end; %Cobb-Douglas
     end;
     %Save end of 12 months of values 
     if (t==months*floor(t/months)); 
     cap_tf(year,:)=single(sum(reshape(cap_now,nop,firms*macro),1)); lab_tf(year,:)=single(sum(reshape(lab_now,nop,firms*macro),1));  sales_tf(year,:)=single(sum(reshape(sales,nop,firms*macro),1)); capexr_tf(year,:)=single(sum(reshape(capexr,nop,firms*macro),1)); 
     end;
             %Analysis
             if simulation==1;
             if t>1;xx=t;
                 ssales1=ssales; ssales=(yyy_now.*((cap_now1./yyy_now).^cap).*((lab_now1./yyy_now).*hours).^lab)./period;
                 output=(yyy_now.^((1-cap-lab)/(cap+lab)).*(cap_now1).^(cap/(cap+lab)).*(lab_now1.*hours).^(lab/(cap+lab)))./period; %Note, ignoring B, that A=Y^(e/(1-e))=Y^((1-cap-lab)/(cap+lab))
                 cap_nowf1=cap_nowf; cap_nowf=single(sum(reshape(cap_now,nop,firms*macro),1));
                 lab_nowf1=lab_nowf; lab_nowf=single(sum(reshape(lab_now,nop,firms*macro),1));
                 sales_nowf1=sales_nowf; sales_nowf=single(sum(reshape(ssales,nop,firms*macro),1));
             sales_mt(xx,:)=sum(double(ssales)); %Cobb-Douglas
             output_mt(xx,:)=sum(double(output)); %Cobb-Douglas
             yyy_mt(xx,:)=sum(double(yyy_now));
             cap_mt(xx,:)=sum(double(cap_now));
             lab_mt(xx,:)=sum(double(lab_now));
             hours_mt(xx,:)=mean(double(hours));
             var_mt(xx,:)=mean(double(varp));
             share1=share; share=double(lab_now1)/sum(double(lab_now1)); TFP_lag=TFP_current;TFP_current=double(share)*double(yyy_now');
             tfp_mt(xx,:)=TFP_current;
             dtfp_t(xx,:)=100*(TFP_current-TFP_lag)/TFP_lag;
             dtfp_w(xx,:)=100*double(share1)*(double(yyy_now)-double(yyy_now1))'/TFP_lag;
             dtfp_b(xx,:)=100*((double(share)-double(share1))*double(yyy_now1'))/TFP_lag;
             dtfp_r(xx,:)=100*(double(share)-double(share1))*double(yyy_now)'/TFP_lag;
             xdtfp_t(xx,:)=100*(double(share)*double(log(yyy_now'))-double(share1)*double(log(yyy_now1')));
             xdtfp_w(xx,:)=100*double(share1)*(double(log(yyy_now))-double(log(yyy_now1)))';
             xdtfp_r(xx,:)=100*(double(share)-double(share1))*double(log(yyy_now))';
             if GE==0;
             if  t==round(period*(ergodic)-1);    yyy_pre2=yyy_now;    lab_pre2=single(lab_now);    end;
             if  t==round(period*(ergodic)+0);    yyy_pre1=yyy_now;    lab_pre1=single(lab_now);    end; 
             if  t==round(period*(ergodic)+1);    yyy_post0=yyy_now;   lab_post0=single(lab_now);   end;
             end;
             end;
             end;
             if simulation==1; disp(t); end;
end;

if simulation==1; tfp(1:10)=ones(10,1); for i=11:t; tfp(i)=tfp(i-1)*(1+(dtfp_r(i)+mean(dtfp_w(10:end)))/100); end; solow(3:t)=output_mt(3:t)./((cap_mt(2:t-1).^(cap/(cap+lab))).*((lab_mt(2:t-1).*hours_mt(3:t)).^(lab/(cap+lab)))); clear varip varip1 yyy_now1 varp ssales ssales1 shockp share share1 muip muip1 lab_now1 index hours1 dyp dym dyf cap_now1 ln gn;
end;
clear sales output capexr

stop;  

%THE COMMANDS PRODUCE PLOTS OF THE SIMULATION OUTPUT

%%FIGURE 4
close;m=2; plot(log(gup(:,round(varnum/2),m)'./h0),log(gup(:,round(varnum/2),m)),'k',log(gdown(:,round(varnum/2),m)'./h0),log(gdown(:,round(varnum/2),m)),'k',log(g0'./hup(:,round(varnum/2),m)),log(g0),'k',log(g0'./hdown(:,round(varnum/2),m)),log(g0),'k','linewidth',3);  axis([-4,1,-6,0]);set(gca,'XTick',-12:1:12); %Need to clean edges where hits end of grid before plotting
%%FIGURE 5
close;m=2; plot(log(gup(:,round(varnum/2),m)'./h0),log(gup(:,round(varnum/2),m)),'k',log(gdown(:,round(varnum/2),m)'./h0),log(gdown(:,round(varnum/2),m)),'k',log(g0'./hup(:,round(varnum/2),m)),log(g0),'k',log(g0'./hdown(:,round(varnum/2),m)),log(g0),'k',log(gup(:,varnum,m)'./h0),log(gup(:,varnum,m)),'--k',log(gdown(:,varnum,m)'./h0),log(gdown(:,varnum,m)),'--k',log(g0'./hup(:,varnum,m)),log(g0),'--k',log(g0'./hdown(:,varnum,m)),log(g0),'--k','linewidth',3);  axis([-4,1,-6,0]);set(gca,'XTick',-12:1:12); %Need to clean edges where hits end of grid before plotting
%%FIGURE 6
muip=single(reshape(repmat(muif,[nop,1]),1,[])); varip=single(reshape(repmat(varim,[nop*firms,1]),1,[])); muvalue=2;gplot=g./((1+inve).*(1-dept)); ln=min(max(1+round((log(l)-log(l0_detail(1)))/lincd),1),lnumd);modeln=mode(ln); ghigh=gplot.*l0_detail(modeln)+100.*((varip~=1)+  (muip~=muvalue)+(ln~=modeln)); [n y]=hist(log(ghigh),150);close;plot(log(g0_detail.*l0_detail(modeln)),estargl(:,modeln,1,muvalue)','k',y,50*n/sum(n(1:100)),'--k','linewidth',2.5,'markersize',2); axis([-3.5,0.5,-1,4]); set(gca,'XTick',-12:1:12,'YTick',-12:1:12); clear muip varip ln gplot ghigh %Plot with high drift. Need to clean distribution plot on right, which comes from hiding the non-fit values (may be easier plot method in matlab)
%Main plot graphs
%%FIGURE 7a
close;plot([0:25]-12,var_mt(109:134)/var_mt(119),'x-k','linewidth',1.5);set(gca,'XTick',-12:6:12); axis([-12,12,0.9,2])
%%FIGURE 7b
close;plot([0:25]-12,yyy_mt(109:134)/yyy_mt(121)+1-(1+mean(mu0p)+0.00005).^([0:25]'-12),'x-k',[0:25]-12,yyy_mt(109:134)/yyy_mt(121),'x-r','linewidth',1.5);set(gca,'XTick',-12:6:12), axis([-12,12,0.98,1.02]); %Note, need to detrend
%%FIGURE 8
close;plot([0:25]-12,lab_mt(108:133)/lab_mt(120)+1-(1+1.34*mean(mu0p)).^([0:25]'-12),'x-k','linewidth',1.5);set(gca,'XTick',-12:6:12), axis([-6,12,0.98,1.02]); %Note, need to detrend
%%FIGURE 9
load VO_output; load SD_output; %Note have to create these somewhere else and load in
close;plot([0:49]-12,SDlab_mt(108:157)/SDlab_mt(120)+1-(1+0.895*mean(mu0p)).^([0:49]'-12),'d-b',[0:49]-12,VOlab_mt(108:157)/VOlab_mt(120)+1-(1+1.3*mean(mu0p)).^([0:49]'-12),'x-r',[0:49]-12,lab_mt(108:157)/lab_mt(120)+1-(1+1.32*mean(mu0p)).^([0:49]'-12),'P-K','linewidth',1.5);set(gca,'XTick',-12:6:36), axis([-6,36,0.98,1.02]) %, set(gca,'YTick',0.98:0.01:1.02);
%%FIGURE 10a
%dtfp_b=dtfp_r+dtfp_c;   %Need to expand size of simulation dataset here to get more smoothing. Try 750000 macro runs as second moments in data take longer to converge
close;plot([0:25]-12,dtfp_r(109:134),'x-r',[0:25]-12,(dtfp_w(110:135)+dtfp_w(108:133)+dtfp_w(109:134))/3,'d-b',[0:25]-12,(dtfp_w(110:135)+dtfp_w(108:133)+dtfp_w(109:134))/3+dtfp_r(109:134),'p-k','linewidth',1.5);set(gca,'XTick',-12:6:12);axis([-12,12,0,4]);%
%%FIGURE 10b
close;plot((yyy_pre1./lab_pre2)',(lab_pre1./lab_pre2)','.k','markersize',5); axis([0,1.8,0,4.5])%Distribution before shock
%%FIGURE 10c
close;plot((yyy_post0./lab_pre1)',(lab_post0./lab_pre1)','.k','markersize',5); axis([0,1.8,0,4.5])%Distribution after shock
%%FIGURE 11
close;plot([0:25]-12,solow(109:134)./solow(121)+1-(1+2.49*mean(mu0p)).^([0:25]-12),'x-k','linewidth',1.5);set(gca,'XTick',-12:6:12),axis([-6,12,0.99,1.01]); %Note, need to detrend
%%FIGURE 12
close;plot([0:25]-12,output_mt(109:134)/output_mt(121)+1-(1+3.8*mean(mu0p)).^([0:25]'-12),'x-k','linewidth',1.5);set(gca,'XTick',-12:6:12),axis([-6,12,0.98,1.02]); %Note, need to detrend
%%FIGURE 14
load GE_output;; %GE Effects
close;plot([0:25]-12,output_mt(109:134)/output_mt(121)+1-(1+3.89*mean(mu0p)).^([0:25]'-12),'x-r',[0:25]-12,GEoutput_mt(109:134)/GEoutput_mt(121)+1-(1+3.835*mean(mu0p)).^([0:25]'-12),'d-k','linewidth',1.5);set(gca,'XTick',-12:6:12), axis([-6,12,0.98,1.02])
%%FIGURE 15
load FM_output; load FO_output; %First and Second moments
close;plot([0:49]-12,output_mt(109:158)/output_mt(121)+1-(1+3.87*mean(mu0p)).^([0:49]'-12),'x-r',[0:49]-12,FOoutput_mt(109:158)/FOoutput_mt(121)+1-(1+3.859*mean(mu0p)).^([0:49]'-12),'x-b',[0:49]-12,FMoutput_mt(109:158)/FMoutput_mt(121)+1-(1+3.89*mean(mu0p)).^([0:49]'-12),'x-k','linewidth',1.5);set(gca,'XTick',-12:6:36), axis([-6,36,0.97,1.02])
%%FIGURE 16
load FC_output; load PI_output; load QA_output; %Different adjustment costs
close;plot([0:49]-12,QAoutput_mt(109:158)/QAoutput_mt(121)+1-(1+2.355*mean(mu0p)).^([0:49]'-12),'x-r',[0:49]-12,FCoutput_mt(109:158)/FCoutput_mt(121)+1-(1+3.53*mean(mu0p)).^([0:49]'-12),'d-b',[0:49]-12,PIoutput_mt(109:158)/PIoutput_mt(121)+1-(1+3.67*mean(mu0p)).^([0:49]'-12),'p-k','linewidth',1.5);set(gca,'XTick',-12:6:36), axis([-6,36,0.98,1.02]) %, set(gca,'YTick',0.98:0.01:1.02);
%%FIGURE 17
load c2_output; load c3_output; load c4_output; load c5_output; load output_baseline; load output_weekly; load output_capitalme; load output_boom; %Simulation robustness
close;plot([0:19]-6,c2output_mt(115:134)/c2output_mt(121)+1-(1+4.05*mean(mu0p)).^([0:19]'-6),'p-r',[0:19]-6,c3output_mt(115:134)/c3output_mt(121)+1-(1+3.95*mean(mu0p)).^([0:19]'-6),'d-k',[0:19]-6,c4output_mt(115:134)/c4output_mt(121)+1-(1+2.85*mean(mu0p)).^([0:19]'-6),'o-g',[0:19]-6,c5output_mt(115:134)/c5output_mt(121)+1-(1+3.95*mean(mu0p)).^([0:19]'-6),'x-b',[0:19]-6,output_baseline_mt(115:134)/output_baseline_mt(121)+1-(1+3.85*mean(mu0p)).^([0:19]'-6),'d--r',[0:19]-6,output_boom_mt(115:134)/output_boom_mt(121)+1-(1+3.95*mean(mu0p)).^([0:19]'-6),'p--k',[0:19]-6,output_capitalme_mt(115:134)/output_capitalme_mt(121)+1-(1+4.2*mean(mu0p)).^([0:19]'-6),'d--b','linewidth',1.5,'markersize',6);set(gca,'XTick',-12:6:12), axis([-6,12,0.97,1.02])
%FIGURE 18
load SS_output; load LS_output; %Length of shock
close;plot([0:49]-12,output_mt(109:158)/output_mt(121)+1-(1+3.87*mean(mu0p)).^([0:49]'-12),'x-r',[0:49]-12,SSoutput_mt(109:158)/SSoutput_mt(121)+1-(1+3.75*mean(mu0p)).^([0:49]'-12),'d-b',[0:49]-12,LSoutput_mt(109:158)/LSoutput_mt(121)+1-(1+3.87*mean(mu0p)).^([0:49]'-12),'p-k','linewidth',1.5);set(gca,'XTick',-12:6:36), axis([-6,36,0.965,1.02]) %, set(gca,'YTick',0.98:0.01:1.02)
%FIGURE 19
load BV_output; load SV_output; %Size of shock
close;plot([0:49]-12,output_mt(109:158)/output_mt(121)+1-(1+3.87*mean(mu0p)).^([0:49]'-12),'x-r',[0:49]-12,SVoutput_mt(109:158)/SVoutput_mt(121)+1-(1+3.735*mean(mu0p)).^([0:49]'-12),'d-b',[0:49]-12,BVoutput_mt(109:158)/BVoutput_mt(121)+1-(1+3.87*mean(mu0p)).^([0:49]'-12),'p-k','linewidth',1.5);set(gca,'XTick',-12:6:36), axis([-6,36,0.975,1.025]) %, set(gca,'YTick',0.98:0.01:1.02)
%FIGURE 20
load output_noplantnofirm; load output_baseline
close;plot([0:25]-12,output_noplantnofirm_mt(109:134)/output_noplantnofirm_mt(121)+1-(1+3.675*mean(mu0p)).^([0:25]'-12),'x-k',[0:25]-12,output_baseline_mt(109:134)/output_baseline_mt(121)+1-(1+3.85*mean(mu0p)).^([0:25]'-12),'o-b','linewidth',1.5);set(gca,'XTick',-12:6:12),axis([-6,12,0.98,1.02]);

%Appendix
%FIGURE A4
load AR_simulation; %AR simulation plot
close; plot(quarter,OUTPUT_DT,'x-k','LineWidth',1.5);;set(gca,'XTick',-12:6:12),axis([-6,12,0.975,1.02]); set(gca,'YTick',0.98:0.01:1.02)
%FIGURE A5
load OFFSET  %gup_OFFSET=gup; gdown_OFFSET=gdown; hup_OFFSET=hup; hdown_OFFSET=hdown; save ('OFFSET','gdown_OFFSET','gup_OFFSET','hdown_OFFSET','hup_OFFSET');
close;m=2; plot(log(gup_OFFSET(:,round(varnum/2),m)'./h0),log(gup_OFFSET(:,round(varnum/2),m)),'k',log(gdown_OFFSET(:,round(varnum/2),m)'./h0),log(gdown_OFFSET(:,round(varnum/2),m)),'k',log(g0'./hup_OFFSET(:,round(varnum/2),m)),log(g0),'k',log(g0'./hdown(:,round(varnum/2),m)),log(g0),'k',log(gup_OFFSET(:,varnum,m)'./h0),log(gup_OFFSET(:,varnum,m)),'.-k',log(gdown_OFFSET(:,varnum,m)'./h0),log(gdown_OFFSET(:,varnum,m)),'.-k',log(g0'./hup_OFFSET(:,varnum,m)),log(g0),'.-k',log(g0'./hdown_OFFSET(:,varnum,m)),log(g0),'.-k','linewidth',1.5);  axis([-4,1,-6,0]);set(gca,'XTick',-12:1:12); %Need to clean edges where hits end of grid before plotting
%FIGURE A6
close;plot([0:25]-12,cap_mt(108:133)/cap_mt(120)+1-(1+mean(mu0p)+0.0008).^([0:25]'-12),'x-k','linewidth',1.5);set(gca,'XTick',-12:6:12), axis([-6,12,0.98,1.02]); %Note, need to detrend
%FIGURE A7
load NACGE_output; %GE without adjustment costs
close;plot([0:48]-12,NACGEoutput_mt(109:157)/NACGEoutput_mt(121)+1-(1+3.6*mean(mu0p)).^([0:48]'-12),'x-k',[0:48]-12,NACGEcap_mt(108:156)/NACGEcap_mt(120)+1-(1+1*mean(mu0p)).^([0:48]'-12),'x-r',[0:48]-12,NACGElab_mt(108:156)/NACGElab_mt(120)+1-(1+1*mean(mu0p)).^([0:48]'-12),'x-b','linewidth',1.5);set(gca,'XTick',-12:6:36); axis([-6,12,0.9,1.5])

%Note: to output into powerpoint I use the following command: print examplefigure -djpeg



