%                   NICK BLOOM (STANFORD) UNCERTAINTY SHOCK SIMULATION (VALUE FUNCTION ITERATION AND NUMERICAL SIMULATION)
%                   THIS VERSION AUGUST 2007.
%                   IF YOU HAVE ANY QUESTIONS OR FIND ANY BUGS PLEASE CONTACT ME AT NBLOOM@STANFORD.EDU, 650-725-7836, THANKS NICK    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Usuage Notes:
% 1) This file is best viewed in Matlab onscreen as the color coding makes it much easier to view and the lines are not "wrapped"
% 2) The program normally runs on Matlab 7.0 for Linux in 64bit using 8GB of memory. If you get an "out of memory" command try reducing
%    the number of points in the state space by lowering the value of "gnum" and reducing the number macro loops ("macro") or plants ("nop")
%    in the simulation.
% 3) This file was created and run in Linux Matlab 7.4. R2007a Some commands may need to be changed for older versions (note my experience is
%    that Linux is worth the pain (in terms of compatability etc..) because it is faster and seems to never crash....
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   STAGE 1 - DEFINING THE PARAMETERS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%Next two lines commented out for interactive (simulation) mode
function criterion=value(inputa,inputb,inputc,inputd,inpute,inputf,inputg,inputh,inputj,inputk)
global v2 runs g0 h0 l0 tranvarm swopcc swopvar returns_init cc0full g0_long h0_long extra muist must varst varist ccst ccist gst gnst lst lnst cap_now1 lab_now1 yyy_now inve emp varnum ccnum factor g0_detail l0_detail sizeg0_long gldetail indexst firms i0 e0 hours0k hhours vcv1 vcv labfull i0nonzero e0nonzero i0squaredcc e0squared i0negativecc e0negative i0cc simcoeff datacoeff vcv4 h0full v3 criterionfull vcvfull  gup gdown hup hdown istar estar mu_mt sales_mt yyy_mt cap_mt lab_mt dtfp_t dtfp_w dtfp_b dtfp_c cor_t ri_mt var_mt tot years ergodic l g inv_ inv_2 inv_4 emp_ emp_2 emp_4 dsales_ dsales_2 dsales_4 estargl istargl simulation nop gnum hnum column markup enum egap cap lab gstart gfinish period;
warning off all; 
%%%Next two lines commented out for simulation mode, next three for experimenting with estimation mode
%clear;  clear global; params=[0.0146    0.0208         0         0    0.3392    0.0181    0.0605         0    0.4432    2.0093];
%inputa=params(1);inputb=params(2);inputc=params(3);inputd=params(4); inpute=params(5); inputf=params(6); inputg=params(7); inputh=params(8); inputj=params(9); inputk=params(10); runs=1; simulation=0; nop=250; column=1; markup=[]; enum=[]; period=[];%0 for estimation set-up and 1 for simulation set-up
%NICK BLOOM, DECEMEBR 2003, VALUE FUNCATION-mean(emp_2)))^(1/2)),...
%A) Setting parameter values
cap=0.25; lab=0.5; if markup==1; cap=(1/3)*(1/1.2); lab=(2/3)*(1/1.2); end;
fck=inputa; fcl=inputb; lambdak=inputc;lambdal=inputd; sellloss=inpute; hirefire=inputf; mud=inputg; mup=inputh; varscale=inputj; wagelam=inputk;
wage1=(wagelam-1)/wagelam; wage2=1/(1+(wagelam-1)*40^wagelam); %Pick parameter based on getting paid time & 1/2 for one extra shift. This implies a ratio of 1.08333 (6.5/6); (wage1.*(1./48+wage2.*48.^(wagelam-1)))/((wage1.*(1./40+wage2.*40.^(wagelam-1))));
scaling=1/3; %Meaningless scale parameter set so that values for main variables are not too small, as the computer precision starts to matter
r=0.065; dep=0.1; mu=0.02; 
if max(size(period))==0; period=12; end; %Sets 12 months a year unless otherwise specified
GE=0; GEPRICE=0.5/100;GEWAGE=0.3/100; GEINT=1.1;%Include GE size price changes
%GE=1; GEWAGE=25/100; GEINT=7;%Figure A5 price changes
hire=12*0.5*hirefire; fire=hire; %assume hiring/firing margin equally split between hiring and firing, and in terms of years wages (noting average weekly wages are 1)
                
gnum=100; %Ideally have about 500 points here but computer speed limits this to 100. Estimation results seem to be insensitive to local variations around 100 (i.e. compare 120 to 80)
if simulation==1; scale=1; gstart=reallog(0.0033); gfinish=reallog(2); ginc=(gfinish-gstart)/(gnum-1); hstart=log(3*(exp(gstart-gfinish))^(1/scale)); end %Cobb-Douglas, Higher lab number shifts to the right
%if simulation==0; scale=1.33; gstart=reallog(0.00033); gfinish=reallog(5); ginc=(gfinish-gstart)/(gnum-1); hstart=log(6*(exp(gstart-gfinish))^(1/scale)); end %Cobb-Douglas, Higher lab number shifts to the right
if simulation==0; scale=1.5; gstart=reallog(0.001); gfinish=reallog(2.5); if markup==1; gstart=reallog(0.0004); gfinish=reallog(1.5); end; ginc=(gfinish-gstart)/(gnum-1); hstart=log(6*(exp(gstart-gfinish))^(1/scale)); if column==4; scale=2; gnum=140; gstart=reallog(0.02); gfinish=reallog(0.5);  ginc=(gfinish-gstart)/(gnum-1); hstart=log(1*(exp(gstart-gfinish))^(1/scale)); end; end %Cobb-Douglas, Higher lab number shifts to the right
if max(size(enum))==0; enum=0; end; egap=(1-exp(enum*-ginc)); %How much extra labor depreciates over capital each period
%gnum=30; scale=0.5;   gstart=reallog(0.5); gfinish=reallog(1.25); ginc=(gfinish-gstart)/(gnum-1); %Labor only
hnum=round(gnum/scale);minhour=25;maxhour=55; %Cap hours at extreme values

hinc=ginc; hfinish=(hnum-1)*hinc+hstart; dept=(1+dep)^(1/period)-1;
var0=(inputj*[1,2]).^2; varnum=2; swopvar=[1-1/36 1/36;0.29 0.71]; if period==1; swopvar=swopvar*swopvar*swopvar*swopvar*swopvar*swopvar*swopvar*swopvar*swopvar*swopvar*swopvar*swopvar; end;
mu0=[mu-mud,mu+mud]; munum=2; swopmu=[1-mup mup; mup 1-mup];       if period==1; swopmu=swopmu*swopmu*swopmu*swopmu*swopmu*swopmu*swopmu*swopmu*swopmu*swopmu*swopmu*swopmu; end;


%if runs==1;
%Setting up the shock transition matrix called tranvarm which is indexed by (gnow,gnext,sigma,mu)
     extra=2*round(gnum/4);
     g0=exp(gstart:ginc:gfinish);h0=exp(hstart:hinc:hfinish);
     g0_long=[exp(gstart-(extra/2)*ginc:ginc:gstart-ginc),g0,exp(gfinish+ginc:ginc:gfinish+(extra/2)*ginc)]; sizeg0_long=size(g0_long,2);
     h0_long=[exp(hstart-(extra/2)*hinc:hinc:hstart-hinc),h0,exp(hfinish+hinc:hinc:hfinish+(extra/2)*hinc)];
     i0=single(((repmat(permute(repmat(g0,[gnum 1])./repmat(g0',[1 gnum]),[1 3 2 4]),[1 hnum 1 hnum])-1)));
     e0=single(((repmat(permute(repmat(h0',[1 hnum])./repmat(h0,[hnum 1]),[3 1 4 2]),[gnum 1 gnum 1])./permute(repmat(repmat(g0',[1 gnum])./repmat(g0,[gnum 1]),[1 1 hnum hnum]),[1 3 2 4])-1 )))+egap;
      factor=repmat(single(((1-dept)/((1+r.*permute(repmat([1 (1-(GEINT/100)/r)]',[1 gnum hnum gnum hnum]),[ 2 3 4 5 1])).^(1/period))).*repmat(1+i0,[1 1 1 1 varnum])),[1 1 1 1 1 munum]); %GE
     gldetail=4; glextra=2*round(gnum/10); gincd=ginc/gldetail; hincd=hinc/gldetail;l0=1./h0; %lnum=hnum; 
     g0_detail=[exp(gstart-(glextra/2)*gincd:gincd:gstart-gincd),exp(gstart:gincd:gfinish),exp(gfinish+gincd:gincd:gfinish+(glextra/2)*gincd)];
     l0_detail=1./[exp(hstart-(glextra/2)*hincd:hincd:hstart-hincd),exp(hstart:hincd:hfinish),exp(hfinish+hincd:hincd:hfinish+(glextra/2)*hincd)];
     i0nonzero=round(1.e+3*i0)~=0; e0nonzero=round(1.e+3*(e0))~=0; 
%end;
     hours0=single(max(min((scaling*lab/(wage1*wage2*wagelam)).^(1/(wagelam-lab)).*((repmat(permute(shiftdim(g0.^((1-cap-lab)/(wagelam-lab)),-2),[1 2 4 3]),[gnum hnum 1 hnum]))).*((repmat(permute(shiftdim(h0.^((lab-1)/(wagelam-lab)),-2),[1 2 3 4]),[gnum hnum gnum 1]))),maxhour),minhour));    
     hours0k=hours0.^wagelam; hhours=permute(hours0(1:varnum,1:munum,:,:,1),[3 4 1 2]);
     %RETURNS per period (month). Dimensions are (hours, investment, employment, capital, labour, uncertainty, drift)
     returns_init =single((scaling/period)*((single(permute(repmat(g0',[1 hnum gnum hnum]),[3 2 1 4])))).^(1-cap-lab).*(((permute(repmat(h0,[gnum 1 gnum hnum]),[1 4 3 2]))).*hours0).^lab);  %Cobb-Douglas
     %returns_init =single((1/period)*(((single(permute(repmat(g0',[1 hnum gnum hnum]),[3 2 1 4])))).^cesparam + (((permute(repmat(h0,[gnum 1 gnum hnum]),[1 4 3 2]))).*hours0).^cesparam)).^((cap+lab)/cesparam);  %CES

for s=1:varnum; for i=1:gnum; for m=1:munum;
     %setting up the values for within period mean and variance - but first correct for linear to log change, using value at sample average
     %gs=g0(gnum/2);loglinear=((normcdf(g0.*(1+ginc/2),gs,gs*(var0(1)/period).^(1/2))-normcdf(g0.*(1-ginc/2),gs,gs*(var0(1)/period).^(1/2)))*log(g0') - (normcdf(g0.*(1+ginc/2),gs,gs*(var0(2)/period).^(1/2))-normcdf(g0.*(1-ginc/2),gs,gs*(var0(2)/period).^(1/2)))*log(g0'))/((var0(2)/period)-(var0(1)/period)); %Simulation estimate of var correction        
     mugt=(1+mu0(m)+dep-0.5*var0(s))^(1/period)-1; sdgt=((var0(s))/period)^(1/2);  %Corrected value with -var/2 log corection (so that in levels drift is mu)
     tranvarm(i,:,s,m)=normcdf(log(g0_long)+ginc/2,mugt+log(g0_long(1))+(i+extra/2-1)*ginc,sdgt)-normcdf(log(g0_long)-ginc/2,log(g0_long(1))+mugt+(i+extra/2-1)*ginc,sdgt);
end; end; end; tranvarm=single(tranvarm);
%Generate monster transition matrix for the one-step expectations calculation - dimension (gnum,hnum,varnum,munum,gnum+extra,varnum,munum), structure (gnow,.,varnow,munow,gnext,varnext,munext)
monster_tran=repmat(permute(tranvarm,[1 5 3 4 2 6 7]),[1 hnum 1 1 1 varnum munum]).*repmat(permute(swopvar,[3 5 1 6 4 2 7]),[gnum hnum 1 munum gnum+extra 1 munum]).*repmat(permute(swopmu,[3 5 6 1 4 7 2]),[gnum hnum varnum 1 gnum+extra varnum 1]);

%Dimension (gnext,hnext,gnow,hnow)
returns=single(repmat(returns_init,[1 1 1 1 varnum]).*permute(repmat([1 1-GE*(GEPRICE)]',[1 gnum hnum gnum hnum]),[2 3 4 5 1]).*(1-(period*fck)*repmat(i0nonzero ,[1 1 1 1 varnum]) - (period*fcl)*repmat(e0nonzero,[1 1 1 1 varnum]))-   (((repmat(permute(h0/period,[3 4 1 2 5]),[gnum hnum gnum 1 varnum]))).*wage1.*(1+wage2.*repmat(hours0k,[1 1 1 1 varnum])).*(permute(repmat([1 1-GE*(GEWAGE)]',[1 gnum hnum gnum hnum]),[2 3 4 5 1])) + repmat(permute(h0,[3 4 1 2 5]),[gnum hnum gnum 1 varnum]).*(repmat(e0,[1 1 1 1 varnum]).*(hire -(fire+hire).*(repmat(e0,[1 1 1 1 varnum])<0)) + lambdal*(period/12)*repmat(e0,[1 1 1 1 varnum]).^2))      -  (repmat(i0,[1 1 1 1 varnum]).*(1 - sellloss.*(repmat(i0,[1 1 1 1 varnum])<0)).*permute(repmat([1 1-0*(GEPRICE)]',[1 gnum hnum gnum hnum]),[2 3 4 5 1]) + lambdak*(period/12).*repmat(i0,[1 1 1 1 varnum]).^2)); %GE
returns=repmat(returns,[1 1 1 1 1 munum]); %sizing up

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   STAGE 2 - RUNNING THE NUMERICAL CONTRACTION MAPPING (SOLVING THE VALUE MAXIMISATION ITERATION)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%STAGE 2 - RUNNING THE VALUE MAXIMISATION ITERATION
v1=single((1-lab)/cap*ones(gnum,hnum,varnum,munum)); %Use fresh value every time for consistency around iterations (i.e. do not use last periods value function)     
Total=150*(period/12); %note - policy correspondences converge very fast (basically completely by 150 loops) and much faster then value functions


for n=1:Total;
%    keyboard;
%Linear extrapolates using result that in adjustement zone value is linear with fixed and/or partial irreversibility, and approximately linear with moderate quadratic adjustment costs
for is=1:varnum; for ic=1:munum; for jh=1:hnum; v1_extrap(:,jh,is,ic)=[(v1(2,jh,is,ic)-v1(1,jh,is,ic)).*((g0_long(1:extra/2)-g0(1))/(g0(2)-g0(1))) + v1(1,jh,is,ic),v1(:,jh,is,ic)',(v1(end,jh,is,ic)-v1(end-1,jh,is,ic)).*((g0_long(gnum+extra/2+1:end)-g0(end))/(g0(end)-g0(end-1))) + v1(end,jh,is,ic)]; end; end; end;

%Multiply together monster transition and monster value function, first create monster value function
monster_value=repmat(permute(v1_extrap,[5 2 6 7 1 3 4]),[gnum 1 varnum munum 1 1 1]);
Ev1=dot(monster_tran(:,:,:,:,:),monster_value(:,:,:,:,:),5);

%The contraction mapping stage
if n<Total;
     %This single line accounts for about 60% of the run time. The order and selection of commands has been chosen  to optimise run speed.
     v1=squeeze(max(max(returns +  factor.*repmat(permute(Ev1,[1 2 5 6 3 4]),[1 1 gnum hnum 1 1]))));
else end;
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   STAGE 3:GENERATING OPTIMAL POLICY CORRESPONDENCE DATA (THE OPTIMAL INVESTMENT FUNCTIONS ETC.)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Solving for optimal investment and employment. Vectorize to avoid looping as much faster
[m1 i1]=max(returns +  factor.*repmat(permute(Ev1,[1 2 5 6 3 4]),[1 1 gnum hnum 1 1]));
%clear returns;
[m2 i2s]=max(squeeze(m1));
v2=squeeze(m2); clear m1 m2;
i2s=reshape(i2s,gnum*hnum*varnum*munum,1);
i1s=i1(i2s+[0:gnum*hnum*varnum*munum-1]'.*hnum); clear i1;
gi=single(reshape(repmat([1:gnum],[1 hnum varnum munum]), gnum*hnum*varnum*munum,1));
hi=single(reshape(repmat(ones(gnum,1)*[1:hnum],[1 1 varnum munum]), gnum*hnum*varnum*munum,1));
istar=single(reshape(g0(gi)./g0(i1s)-1,[gnum hnum varnum munum]));
estar=single(reshape((g0(gi)./g0(i1s)).*(h0(i2s)./h0(hi))-1,[gnum hnum varnum munum]));
gstar=single(reshape(g0(i1s),[gnum hnum varnum munum]));

%Edit this out when running the actual simulation as only used for looking at results
hstar=single(reshape(h0(i2s).*(g0(gi)./g0(i1s)),[gnum hnum varnum munum]));
hoursstar=min(max(single((scaling*lab/(wage1*wage2*wagelam)).^(1/(wagelam-lab)).*(repmat(g0',[1 hnum varnum munum])).^((1-cap-lab)/(wagelam-lab)).*((repmat(h0,[gnum 1 varnum munum])).^((lab-1)/(wagelam-lab)))),minhour),maxhour);
     
gup=[];gdown=[]; g0=single(g0); h0=single(h0);
for bh=1:hnum; for cs=1:varnum; for cc=1:munum;
m=max(g0'.*(g0'==gstar(:,bh,cs,cc))); gup(bh,cs,cc)=m; 
m=min(g0'.*(g0'==gstar(:,bh,cs,cc))+g0(gnum).*(1-(g0'==gstar(:,bh,cs,cc)))); gdown(bh,cs,cc)=m; 
end; end; end;
hup=[];hdown=[];for bh=1:gnum; for cs=1:varnum; for cc=1:munum;
m=max(h0.*(h0*(1-egap)==hstar(bh,:,cs,cc)));hup(bh,cs,cc)=m;
m=min(h0.*(h0*(1-egap)==hstar(bh,:,cs,cc))+h0(hnum).*(1-(h0*(1-egap)==hstar(bh,:,cs,cc)))); hdown(bh,cs,cc)=m;
end; end; end;

%NEED TO SPEED UP THIS EXTRAPOLATION - SHOULD REALLY VECTORIZE THIS PART BUT NOT THAT SLOW SO LEFT FOR PRESENT
gnumd=length(g0_detail); lnumd=length(l0_detail);
for s=1:varnum; for cc=1:munum; for h=1:hnum; 
                                             istarg(:,h,s,cc)=interp1(g0',istar(:,h,s,cc),g0_detail','linear','extrap');
                                             estarg(:,h,s,cc)=interp1(g0',estar(:,h,s,cc),g0_detail','linear','extrap');
                                             hoursg(:,h,s,cc)=interp1(g0',hhours(:,h,s,cc),g0_detail','linear','extrap');
                                             valueg(:,h,s,cc)=interp1(g0',v1(:,h,s,cc),g0_detail','linear','extrap'); 
                                             end;end;end;
                                 
for s=1:varnum; for cc=1:munum; for g=1:gnumd;
                                             istargl(g,:,s,cc)=interp1(l0',istarg(g,:,s,cc),l0_detail','linear','extrap');
                                             estargl(g,:,s,cc)=interp1(l0',estarg(g,:,s,cc),l0_detail','linear','extrap');
                                             hoursgl(g,:,s,cc)=interp1(l0',hoursg(g,:,s,cc),l0_detail','linear','extrap');
                                             valuegl(g,:,s,cc)=interp1(h0',valueg(g,:,s,cc),l0_detail','linear','extrap'); 
                                             end; end; end;
estargl=min(max(estargl,-0.8),10); istargl=min(max(istargl,-0.8),5); hoursgl=min(max(hoursgl,minhour),maxhour); %safety precaution against extrapolation on extreme values
%}

%Clear some unnecessary variables before start the simulation
clear ystarg ystargl lstarg lstargl gstarg gstargl istarg estarg ykstargl ylstargl ykstarg ylstarg v2_extrap valueg cashg temp tranvarm tranvarm_g returns index index1 index2 vv2 i0full e0full returns1 h0full g0full inve emp dy cap_t cap_now lab_t lab_now yyy_t yyy_now cap_tf dyhistory cc0full factor returns capexr_tf hiring_tf stdr_tf ihistory ehistory dyhistory var_tf vhistory yyy_y;  
clear cap_mt lab_mt sales_mt yyy_mt ri_mt var_mt mu_mt tfp_mt dtfp* monster_value hours capex* cap_* lab_* sales_* dy* hours1 hours2 yyy_* sd*
%}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   STAGE 4: RUNNING THE DATA SIMULATIONS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
dept=dep/period; ldept=(dep+egap)/period; egapt=egap/period; var0p=(var0/period).^(1/2); mu0p=mu0/period; %Note var0p is really the monthly sd of the shocks
volatility=1; %Allow volatility to change alongside uncertainty 
if simulation==0, macro=10; if max(size(nop))==0; nop=250; end; firms=1000; yearreg=20;  %Note change to macro=25 when running actual estimation
else; macro=25000;  nop=250; firms=1000/nop; yearreg=5; ssales=ones(1,macro*firms*nop); cap_nowf=ones(1,macro*firms); lab_nowf=cap_nowf; sales_nowf=cap_nowf; share=ones(1,macro*firms*nop); TFP_current=1; lab_now2=share;clear returns* factor* e0* i0* monster* hours0*; end;
ergodic=10; rand('state',0); randn('state',0); %Note need 4 lags so an ergodic of 7 provides 3 years before first data collected. Tradeoff in ergodic as larger values take longer and generate more cross-sectional size spread (less aggregation)
gincd=ginc/gldetail; lincd=-hinc/gldetail; 
months=period;
sharem=1/3.^(1/2); sharef=1/3.^(1/2); sharep=1/3.^(1/2); % Share macro,firm and plant of all total variance
years=ergodic+yearreg; 
for t=1:years*period;   %SIMULATION LOOP OVER PERIODS (MONTHS/WEEKS ETC.)
    year=floor(t/months);
  if t==1;    %FIRST PERIOD - THROW AWAY VALUES SO ONLY NEED TO SET UP AS RIGHT DIMENSIONS
%   if runs==1;  %Set up initial state for first period and use the same for all loops
   g=single(exp(log(gdown(round(hnum/2),1,1))+(log(gup(round(hnum/2)-1,1,1))-log(gdown(round(hnum/2),1,1))).*(rand(1,nop*firms*macro).^(1/6)))); if max(isnan(g))==1; g=(g<100); else if mean(g)==0; g=(g<100); end; end;;   %Good guess for initial distribution and then ergodic to run this in for 5 years
   l=single(1./hup(round(gnum/2),1,1)+((1./hdown(round(3*gnum/4),1,1))-(1./hup(round(gnum/2),1,1))).*rand(1,nop*firms*macro)); if max(isnan(l))==1; l=(l<100); else if mean(l)==0; l=(l<100); end; end;;
%   end; %Only for first loop, use earlier values after that
   ccap_now=ones(nop,firms*macro); cap_now=reshape(ccap_now,1,nop*firms*macro);cap_now1=cap_now; clear ccap_now; %Set up equal sizes to start, however
   lab_now=cap_now.*l; lab_now1=lab_now; yyy_now=cap_now.*g;yyy_now1=yyy_now;
   %First setting up firm values for var and cc (both levels and indexes)
   varim=ones(1,macro)+(rand(1,macro)>( swopvar(2,1)/(swopvar(2,1)+swopvar(1,2)) ));  %Set up with ergodic steady-state
   varif=single(reshape(repmat(varim,[firms,1]),1,[]));
   varip=single(reshape(repmat(varim,[nop*firms,1]),1,[]));
   varm=var0p(ones(1,macro)); 
   varf=var0p(varif); 
   varp=var0p(varip); 
   muif=ones(1,firms*macro)+(rand(1,firms*macro)>0.5);  %Set up with ergodic steady-state
   muf=mu0p(muif); 
   muip =single(reshape(repmat(muif,[nop,1]),1,[]));
   %Then moving to plant values for var and cc (both levels and indexes)
   shockm=randn(1,macro); shockf=randn(1,firms*macro); shockp=randn(1,nop*firms*macro);
   dym=sharem*shockm.*varm; dym=reshape(repmat(dym,[nop*firms,1]),1,[]);
   dyf=muf + sharem*shockf.*varf; dyf=reshape(repmat(dyf,[nop,1]),1,[]);
   dyp=sharep*shockp.*varp;
   dy=max(min(dym+dyf+dyp,0.25*(12/period)^(1/2)),-0.25*(12/period)^(1/2));
   g=g.*(1+dy); inve=single(zeros(1,firms*nop*macro)); emp=inve; hours=inve;
   else     %ALL SUBSEQUENT PERIODS
   %Note that: (i) dy varies by unit, firm and panel; (ii) var varies by panel; and (iii) mu varies by firm 
   varshock=single(rand(1,macro));
   varim1=varim; varim=(varim1==1).*(1+(varshock>swopvar(1,1)))+(varim==2).*(2-(varshock<swopvar(2,1)));
   if simulation==1; if  t==round(period*(ergodic)+1);  varim=2*(varim>0); end; end;    %Second moment shock
   %   if  t==round(period*(ergodic + 4.5)); dy=dy+1; end;                              %First moment just
   varif1=varif; varif=single(reshape(repmat(varim,[firms,1]),1,[])); 
   varip1=varip; varip=single(reshape(repmat(varim,[nop*firms,1]),1,[]));
   varm=single(var0p(varim1));  %Note mean and variance depends on lagged values
   varf=single(var0p(varif1)); 
   varp=single(var0p(varip1)); 
   mushock=single(rand(1,firms*macro)); 
   muif1=muif; muif=single((muif1==1).*(1+(mushock>swopmu(1,1)))+(muif1==2).*(2-(mushock<swopmu(2,1))));
   muf=single(mu0p(muif1)); 
   muip1=muip; muip=single(reshape(repmat(muif,[nop,1]),1,[])); 
   shockm=single(randn(1,macro)); shockf=single(randn(1,firms*macro)); shockp=single(randn(1,nop*firms*macro));
   if volatility==0; varm=var0p(1).*(varm>0); varf=var0p(1).*(varf>0); varp=var0p(1).*(varp>0); end; %Imposes constant volatility of actual shocks
   dym=sharem*shockm.*varm;
   dym=single(reshape(repmat(dym,[nop*firms,1]),1,[]));
   dyf=muf + sharem*shockf.*varf; dyf=single(reshape(repmat(dyf,[nop,1]),1,[]));
   dyp=sharep*shockp.*varp;
   dy=max(min(dym+dyf+dyp,0.2*(12/period)^(1/2)+mean(mu0)*(12/period)),-0.2*(12/period)^(1/2)+mean(mu0)*(12/period)); %Cap monthly shocks at 25% to stop process exploding and preserve aggregation effects for longer
   g=g.*(1+dy)./((1+inve).*(1-dept)); l=l.*(1+inve)./((1+emp).*(1-egapt));
   gn=min(max(1+round((log(g)-log(g0_detail(1)))/gincd),1),gnumd);
   ln=min(max(1+round((log(l)-log(l0_detail(1)))/lincd),1),lnumd);   
   index=(gn + (ln-1)*gnumd + (varip-1)*gnumd*lnumd + (muip-1)*gnumd*lnumd*varnum);
   inve=istargl(index);
   emp=estargl(index); if simulation==1; hours1=hours;end
   hours=hoursgl(index);
   end;
     %%%Generating capital and yyy values    
     if simulation==1; lab_now3=lab_now2; lab_now2=lab_now1; end
     cap_now1=cap_now; lab_now1=lab_now; yyy_now1=yyy_now;
     cap_now=double(cap_now1.*(1+inve).*(1-dept)); lab_now=double(lab_now1.*(1+emp).*(1-ldept)); yyy_now=single(yyy_now1.*(1+dy)+1.0e-10); 
     if (t==months*floor(t/months)+1)||period==1; year=floor(t/months); %Generating sales and capex values (note have to reset every 12 months and add over the year)
                 capexr=inve.*(1-(inve<0)*sellloss).*cap_now1;
                 sales=(yyy_now.*((cap_now1./yyy_now).^cap).*((lab_now1./yyy_now).*hours).^lab)./period; %Cobb-Douglas
                 %sales=(yyy_now.*((cap_now1./yyy_now).^cesparam + (lab_now1./yyy_now).^cesparam).^((cap+lab)/cesparam))/period; %Ces
     elseif period>1;
                 capexr=capexr+inve.*(1-(inve<0)*sellloss).*cap_now1;
                 sales =sales +(yyy_now.*((cap_now1./yyy_now).^cap).*((lab_now1./yyy_now).*hours).^lab)./period; %Cobb-Douglas
                 %sales =sale  +(yyy_now.*((cap_now1./yyy_now).^cesparam + (lab_now1./yyy_now).^cesparam).^((cap+lab)/cesparam))/period %CES
     end;
     %Save end of 12 months of values 
     if (t==months*floor(t/months))||period==1; 
     cap_tf(year,:)=single(sum(reshape(cap_now,nop,firms*macro),1)); lab_tf(year,:)=single(sum(reshape(lab_now,nop,firms*macro),1));  sales_tf(year,:)=single(sum(reshape(sales,nop,firms*macro),1)); capexr_tf(year,:)=single(sum(reshape(capexr,nop,firms*macro),1)); 
     end;
             %Analysis
             if simulation==1;
             if t>1;xx=t;
                 ssales1=ssales; ssales=(yyy_now.*((cap_now1./yyy_now).^cap).*((lab_now1./yyy_now).*hours).^lab)./period;
                 output=(yyy_now.^((1-cap-lab)/(cap+lab)).*(cap_now1).^(cap/(cap+lab)).*(lab_now1.*hours).^(lab/(cap+lab)))./period;
                 cap_nowf1=cap_nowf; cap_nowf=single(sum(reshape(cap_now,nop,firms*macro),1));
                 lab_nowf1=lab_nowf; lab_nowf=single(sum(reshape(lab_now,nop,firms*macro),1));
                 sales_nowf1=sales_nowf; sales_nowf=single(sum(reshape(ssales,nop,firms*macro),1));
             sales_mt(xx,:)=sum(double(ssales)); %Cobb-Douglas
             output_mt(xx,:)=sum(double(output)); %Cobb-Douglas
             sdoutput_mt(xx,:)=std(double(output));
             moutput_mt(xx,:)=mean(double(output));
             yyy_mt(xx,:)=sum(double(yyy_now));
             cap_mt(xx,:)=sum(double(cap_now));
             lab_mt(xx,:)=sum(double(lab_now));
             dlab=lab_now-lab_now1; dqlab=lab_now-lab_now3;
             jobcreation(xx,:)=sum(double(dlab.*(dlab>0)))/lab_mt(xx-1); 
             jobdestruction(xx,:)=sum(double(-dlab.*(dlab<0)))/lab_mt(xx-1); 
             qjobcreation(xx,:)=sum(double(dqlab.*(dqlab>0)))/lab_mt(xx-1); 
             qjobdestruction(xx,:)=sum(double(-dqlab.*(dqlab<0)))/lab_mt(xx-1); 
             clear dlab dqlab output;
             hours_mt(xx,:)=mean(double(hours));
             sddsales_mt(xx,:)=std(double(ssales./ssales1)); %Cobb-Douglas
             sdsales_mt(xx,:)=std(double(ssales)); %Cobb-Douglas
             sddyyy_mt(xx,:)=std(double(yyy_now./yyy_now1));
             sdyyy_mt(xx,:)=std(double(yyy_now));
             sddcap_mt(xx,:)=std(double(cap_now./cap_now1));
             sdcap_mt(xx,:)=std(double(cap_now));
             sddlab_mt(xx,:)=std(double(lab_now./lab_now1));
             sdlab_mt(xx,:)=std(double(lab_now));
             share1=share; share=lab_now/sum(lab_now); TFP_lag=TFP_current;TFP_current=share*yyy_now';
             tfp_mt(xx,:)=TFP_current;
             dtfp_t(xx,:)=100*(TFP_current-TFP_lag)/TFP_lag;
             dtfp_w(xx,:)=100*(share1*(yyy_now-yyy_now1)')/TFP_lag;
             dtfp_b(xx,:)=100*((share-share1)*yyy_now1')/TFP_lag;
             dtfp_c(xx,:)=100*((share-share1)*(yyy_now-yyy_now1)')/TFP_lag;
             var_mt(xx,:)=mean(double(varp));
             if GE==0;
             if  t==round(period*(ergodic+5)-2);    yyy_pre2=yyy_now;    lab_pre2=single(lab_now);    end; %cap_pre2=single(cap_now);    end; 
             if  t==round(period*(ergodic+5)-1);    yyy_pre1=yyy_now;    lab_pre1=single(lab_now);    end; %cap_pre1=single(cap_now);    end; 
             if  t==round(period*(ergodic+5));      yyy_post0=yyy_now;   lab_post0=single(lab_now);   end; %cap_post0=single(cap_now);   end; 
             end;
             end;
             end;
             if simulation==1; disp(t); end;
end;

if simulation==1; tfp(1:10)=ones(10,1); for i=11:t; tfp(i)=tfp(i-1)*(1+(dtfp_b(i)+dtfp_c(i)+mean(dtfp_w(10:end)))/100); end; solow(3:t)=output_mt(3:t)./((cap_mt(2:t-1).^(cap/(cap+lab))).*((lab_mt(2:t-1).*hours_mt(3:t)).^(lab/(cap+lab)))); clear varip varip1 yyy_now1 varp ssales ssales1 shockp share share1 muip muip1 lab_now1 lab_now2 index hours1 dyp dym dyf cap_now1 cap_now2 ln gn; stop; end;
tot=(years-ergodic)*period;
lag=2;lagbig=4; laggap=lagbig-lag; 
%sales_tf=sales_tf.*exp(0.01*randn(years,firms*macro));% 1% measurement error in sales
dsales_cur=(sales_tf(2+lagbig:end,:)-sales_tf(1+lagbig:end-1,:))./(0.5*(sales_tf(1+lagbig:end-1,:)+sales_tf(2+lagbig:end,:)));%pu_s=prctile(dsales_cur(:),99.5);pd_s=prctile(dsales_cur(:),0.5); dsales_cur=max(min(dsales_cur,pu_s),pd_s);
dsales_lag=(sales_tf(2+laggap:end-lag,:)-sales_tf(1+laggap:end-1-lag,:))./(0.5*(sales_tf(1+laggap:end-1-lag,:)+sales_tf(2+laggap:end-lag,:)));% dsales_lag=max(min(dsales_lag,pu_s),pd_s);
dsales_lagbig=(sales_tf(2:end-lagbig,:)-sales_tf(1:end-1-lagbig,:))./(0.5*(sales_tf(1:end-1-lagbig,:)+sales_tf(2:end-lagbig,:)));% dsales_lag4=max(min(dsales_lagbig,pu_s),pd_s);
inv_cur=(capexr_tf(2+lagbig:end,:))./(0.5*(cap_tf(1+lagbig:end-1,:)+cap_tf(2+lagbig:end,:)));% pu_i=prctile(inv_cur(:),99.5);pd_i=prctile(inv_cur(:),0.5);  inv_cur=max(min(inv_cur,pu_i),pd_i);
inv_lag=(capexr_tf(2+laggap:end-lag,:))./(0.5*(cap_tf(1+laggap:end-1-lag,:)+cap_tf(2+laggap:end-lag,:))); %inv_lag=max(min(inv_lag,pu_i),pd_i);
inv_lagbig=(capexr_tf(2:end-lagbig,:))./(0.5*(cap_tf(1:end-1-lagbig,:)+cap_tf(2:end-lagbig,:)));% inv_lagbig=max(min(inv_lagbig,pu_i),pd_i);
lab_tfe=lab_tf(end-years+1:end,:).*exp(0.1*randn(years,firms*macro)); %Introducing 10% noise into labour values
dlab_cur=(lab_tfe(2+lagbig:end,:)-lab_tfe(1+lagbig:end-1,:))./(0.5*(lab_tfe(1+lagbig:end-1,:)+lab_tfe(2+lagbig:end,:))); %pu_l=prctile(dlab_cur(:),99.5);pd_l=prctile(dlab_cur(:),0.5);  dlab_cur=max(min(dlab_cur,pu_l),pd_l);
dlab_lag=(lab_tfe(2+laggap:end-lag,:)-lab_tfe(1+laggap:end-1-lag,:))./(0.5*(lab_tfe(1+laggap:end-1-lag,:)+lab_tfe(2+laggap:end-lag,:))); %dlab_lag=max(min(dlab_lag,pu_l),pd_l);
dlab_lagbig=(lab_tfe(2:end-lagbig,:)-lab_tfe(1:end-1-lagbig,:))./(0.5*(lab_tfe(1:end-1-lagbig,:)+lab_tfe(2:end-lagbig,:)));% dlab_lagbig=max(min(dlab_lagbig,pu_l),pd_l);
inv_ =reshape(inv_cur(end-yearreg+1:end,:),firms*(yearreg),macro);
inv_2=reshape(inv_lag(end-yearreg+1:end,:),firms*(yearreg),macro);
inv_4=reshape(inv_lagbig(end-yearreg+1:end,:),firms*(yearreg),macro);
emp_ =reshape(dlab_cur(end-yearreg+1:end,:),firms*(yearreg),macro);
emp_2=reshape(dlab_lag(end-yearreg+1:end,:),firms*(yearreg),macro);
emp_4=reshape(dlab_lagbig(end-yearreg+1:end,:),firms*(yearreg),macro);
dsales_ =reshape(dsales_cur(end-yearreg+1:end,:),firms*(yearreg),macro);
dsales_2=reshape(dsales_lag(end-yearreg+1:end,:),firms*(yearreg),macro);
dsales_4=reshape(dsales_lagbig(end-yearreg+1:end,:),firms*(yearreg),macro);

simcoeff=[
    mean(diag(corr(inv_,inv_2))),...
    mean(diag(corr(inv_,inv_4))),...
    mean(diag(corr(inv_,emp_2))),...
    mean(diag(corr(inv_,emp_4))),...
    mean(diag(corr(inv_,dsales_2))),...
    mean(diag(corr(inv_,dsales_4))),...
    mean(std(inv_)),...
    mean(skewness(inv_)),...
    mean(diag(corr(emp_,inv_2))),...
    mean(diag(corr(emp_,inv_4))),...
    mean(diag(corr(emp_,emp_2))),...
    mean(diag(corr(emp_,emp_4))),...
    mean(diag(corr(emp_,dsales_2))),...
    mean(diag(corr(emp_,dsales_4))),...
    mean(std(emp_)),...
    mean(skewness(emp_)),...
    mean(diag(corr(dsales_,inv_2))),...
    mean(diag(corr(dsales_,inv_4))),...
    mean(diag(corr(dsales_,emp_2))),...
    mean(diag(corr(dsales_,emp_4))),...
    mean(diag(corr(dsales_,dsales_2))),...
    mean(diag(corr(dsales_,dsales_4))),...
    mean(std(dsales_)),...
    mean(skewness(dsales_))];

datacoeff=[0.328,0.258,0.208,0.158,0.260,0.201,0.139,1.789,...
           0.188,0.133,0.160,0.108,0.193,0.152,0.189,0.445,...
           0.203,0.142,0.161,0.103,0.207,0.156,0.165,0.342];           
load v_c_v.csv %var_cov_var=eye(15);
criterion=(simcoeff-datacoeff)*v_c_v*(simcoeff-datacoeff)'; 


%{
%THE COMMANDS PRODUCE PLOTS OF THE SIMULATION OUTPUT
%Cross-sectional histograms to compare to actual moments
[n y]=hist(dsales_cur,100);close;bar(y,100*n/sum(n));;
[n y]=hist(dlab_cur,40);close;bar(y,100*n/sum(n));axis([-1.33,2,0,12]);
[n y]=hist(inv_cur,50);close;bar(y,100*n/sum(n));axis([-0.4,0.8,0,30]);
%Useful for looking at the response space
%Treshold graphs
close;cc=1;m=2; plot(log(gup(:,1,m,cc)'./h0),log(gup(:,1,m,cc)),'g',log(gdown(:,1,m,cc)'./h0),log(gdown(:,1,m,cc)),'g',log(g0'./hup(:,1,m,cc)),log(g0),'g',log(g0'./hdown(:,1,m,cc)),log(g0),'r',log(gup(:,round(varnum/2),m,cc)'./h0),log(gup(:,round(varnum/2),m,cc)),'k',log(gdown(:,round(varnum/2),m,cc)'./h0),log(gdown(:,round(varnum/2),m,cc)),'b');
close;cc=1;m=2; plot(log(gup(:,1,m,cc)'./h0),log(gup(:,1,m,cc)),'g',log(gdown(:,1,m,cc)'./h0),log(gdown(:,1,m,cc)),'g',log(g0'./hup(:,1,m,cc)),log(g0),'g',log(g0'./hdown(:,1,m,cc)),log(g0),'g',log(gup(:,round(varnum/2),m,cc)'./h0),log(gup(:,round(varnum/2),m,cc)),'b',log(gdown(:,round(varnum/2),m,cc)'./h0),log(gdown(:,round(varnum/2),m,cc)),'b',log(g0'./hup(:,round(varnum/2),m,cc)),log(g0),'b',log(g0'./hdown(:,round(varnum/2),m,cc)),log(g0),'b',log(gup(:,varnum,m,cc)'./h0),log(gup(:,varnum,m,cc)),'r',log(gdown(:,varnum,m,cc)'./h0),log(gdown(:,varnum,m,cc)),'r',log(g0'./hup(:,varnum,m,cc)),log(g0),'r',log(g0'./hdown(:,varnum,m,cc)),log(g0),'r');
close;plot(0.65*log(g0)-0.6,estar(:,round(hnum/2)+10,1,1)','b',0.65*log(g0)-0.6,estar(:,round(hnum/2)+10,2,1)','r');
close;plot(0.65*log(g0)-0.6,istar(:,round(hnum/2)+10,1,1)','b',0.65*log(g0)-0.6,istar(:,round(hnum/2)+10,2,1)','r');
%}