set more 1
clear

*******************************************************************************************
*INSHEETS VOLATILITY FIGURES FROM CBOE DATABASE AND MERGES WITH SP500 MEASURE CREATED BELOW
********************************************************************************************

*Generate baseline stock volatility data to merge in later
insheet using sp500_long.csv,c
egen sdri=sd(ri),by(ym)
so ym
gen date=mdy(month,day,year)
so date
save splong,replace


*Raw volatility figures - can update using http://www.cboe.com/micro/vix/historical.aspx
insheet using vxo.csv,c names clear
cap destring vol,force replace
cap gen vol=real(close)
gen slot2=substr(date,2,1)
gen slot3=substr(date,3,1)
gen slot4=substr(date,4,1)
gen slot5=substr(date,5,1)
gen slot6=substr(date,6,1)
gen     month=real(substr(date,1,2)) if slot3=="/"
replace month=real(substr(date,1,1)) if slot2=="/"
gen         day=real(substr(date,4,2))   if slot6=="/"
replace     day=real(substr(date,4,1))   if slot3=="/"&slot5=="/"
replace     day=real(substr(date,3,1))   if slot2=="/"&slot4=="/"
replace     day=real(substr(date,3,2))   if slot2=="/"&slot5=="/"
gen        year=real(substr(date,7,4))   if slot6=="/"
replace    year=real(substr(date,6,4))   if slot5=="/"
replace    year=real(substr(date,5,4))   if slot4=="/"
replace year=year+1900 if year>10&year<100
replace year=year+2000 if year<100
drop date
gen date=mdy(month,day,year)
gen dow=dow(date)
gen m2=month
gen d2=day
replace month=d2 if dow==6|dow==0
replace day  =m2 if dow==6|dow==0
replace date=mdy(month,day,year)
format date %d
drop slot* day
expand 10 if date>15227&date<15235
so date
replace date=date[_n-1]+1 if date==15228&date[_n-1]~=15227
drop dow
gen dow=dow(date)
drop if dow==0|dow==6
so date

*Fill in 9/11 figures using European exchange extraploation
drop if date==date[_n+1]
replace vol=. if date>=15229&date<=15232
replace vol=58.2 if date>15228&date<15234
drop if vol==.
so date

*Drop months with less than 12 days of data
egen dayc=count(month),by(month year)
*drop if dayc<10
drop dayc
gen ym=year+(month-1)/12
so date
*Merge in with standard S&P500 stock returns volatility
merge date using splong
drop if _==2&year>1990
egen mvol=mean(vol),by(year month)
replace sdri=sdri*(366^(1/2))
gen miss=(mvol==.)

*Normalize to have same mean and variance (but not Black-Monday values as so extreme)
replace miss=1 if (year==1987&(month==10|month==11|month==12))|(year==1988&month==1)
replace miss=. if sdri==.
egen sd_mvol=sd(mvol) if miss==0
egen sd_sdri=sd(sdri) if miss==0
egen sdmvol=max(sd_mvol)
egen sdsdri=max(sd_sdri)
replace sdri=sdri*(sdmvol/sdsdri)
egen m_mvol=mean(mvol) if miss==0
egen m_sdri=mean(sdri) if miss==0
egen mmvol=max(m_mvol)
egen msdri=max(m_sdri)
replace sdri=sdri + mmvol-msdri
replace mvol=sdri if mvol==.&year<1987
lab var mvol "                                                                                                             "
so ym
drop if ym==ym[_n-1]

lab var mvol "Volatitily (actual 1880-1985, implied 1986-2008)"
lab var ym "Year"
scatter  mvol ym, c(l) msize(0.75) yaxis(1) xlabel(1880(10)2010) ylabel(10(10)80) legend(off)
lab var mvol " "
