clear
set memory 200m
set matsize 3000


insheet using vardata.csv
drop if ipm==.
gen period=12*year+month
tsset period
gen ym=year+(month-1)/12
gen lipm=log(ipm)
gen lcpi=log(cpi)
gen infl=cpi-cpi[_n-1]
gen lempm=log(empm)
gen lstock=log(stock)
gen loph=log(ipm/(empm*hoursm*cpi))
gen lroph=log(ipm/(empm*hoursm*cpi))
gen lhoursm=log(hoursm)
gen lwage=log(wage)

*HP filters - run line below to install hprescott command
*ssc install hprescott
foreach var of varlist lipm ffr infl lempm lstock volatbl hoursm lhoursm lwage lcpi{

*Standard HP filtering
qui hprescott `var',stub(hp) smooth(129600)
qui gen c`var'=`var'-hp_`var'_sm_1

*Alternative 10 smoothing
cap qui hprescott `var',stub(hp2) smooth(1296)
qui gen hf`var'=`var'-hp2_`var'_sm_1

*Alternative linear detrend smoothing
cap qui hprescott `var',stub(hp3) smooth(99999999)
qui gen ld`var'=`var'-hp3_`var'_sm_1

}

drop if volatbl==.

*Maximum month timing
gen clean=0
replace clean=1 if year==1962&month==10
replace clean=1 if year==1963&month==11
replace clean=1 if year==1966&month==8
replace clean=1 if year==1970&month==5
replace clean=1 if year==1973&month==12
replace clean=1 if year==1974&month==10
replace clean=1 if year==1978&month==11
replace clean=1 if year==1980&month==3
replace clean=1 if year==1982&month==10
replace clean=1 if year==1987&month==11
replace clean=1 if year==1990&month==10
replace clean=1 if year==1997&month==11
replace clean=1 if year==1998&month==9
replace clean=1 if year==2001&month==9
replace clean=1 if year==2002&month==9
replace clean=1 if year==2003&month==2
replace clean=1 if year==2003&month==2
replace clean=1 if year==2008&month==3


*First month timing
gen fclean=0
replace fclean=1 if year==1962&month==10
replace fclean=1 if year==1963&month==11
replace fclean=1 if year==1966&month==8
replace fclean=1 if year==1970&month==5
replace fclean=1 if year==1973&month==11
replace fclean=1 if year==1974&month==7
replace fclean=1 if year==1978&month==11
replace fclean=1 if year==1980&month==3
replace fclean=1 if year==1982&month==8
replace fclean=1 if year==1987&month==10
replace fclean=1 if year==1990&month==8
replace fclean=1 if year==1997&month==11
replace fclean=1 if year==1998&month==9
replace fclean=1 if year==2001&month==9
replace fclean=1 if year==2002&month==7
replace fclean=1 if year==2003&month==2
replace fclean=1 if year==2007&month==8


*Oil & terror 
gen oclean=0
replace oclean=1 if year==1962&month==10
replace oclean=1 if year==1963&month==11
replace oclean=1 if year==1966&month==10
replace oclean=1 if year==1970&month==5
replace oclean=1 if year==1973&month==12
replace oclean=1 if year==1978&month==11
replace oclean=1 if year==1980&month==3
replace oclean=1 if year==1990&month==10
replace oclean=1 if year==2001&month==9
replace oclean=1 if year==2003&month==2

*Other
gen xclean=0
replace xclean=1 if year==1974&month==9
replace xclean=1 if year==1982&month==8
replace xclean=1 if year==1987&month==10
replace xclean=1 if year==1997&month==11
replace xclean=1 if year==1998&month==9
replace xclean=1 if year==2002&month==7

*Scaled by shock size
gen sclean=clean*cvolatbl

save vol,replace
u vol,replace
irf set irf_final,replace


******MAIN BASLINE RESULTS
***Kitchen sink (full specification) - note this is very slow to run because of the standard-error calculations
qui var clstock clean cffr clwage clcpi choursm clempm clipm,lags(1(1)12)
irf create kitchen,step(36) replace set(irf_final)
*irf ograph (kitchen clean clipm oirf), legend(off) xlab(0(6)36)


******VARIABLES AND ORDERING
*Bivariate
qui var clean clstock clempm clipm,lags(1(1)12)
irf create bivariate,step(36) replace set(irf_final) nose
***Trivariate 
qui var clean clstock cffr clipm,lags(1(1)12)
irf create baseline,step(36) replace set(irf_final) nose
***Trivariate reversed
qui var clipm cffr clstock clean,lags(1(1)12)
irf create reverse,step(36) replace set(irf_final) nose

******SHOCK MEASURES
***Kitchen sink - oil,war & terror only
qui var clstock oclean cffr clwage clcpi choursm clempm clipm,lags(1(1)12)
irf create okitchen,step(36) replace set(irf_final) nose
***Kitchen sink - actual volatility data
qui var clstock volatbl cffr clwage clcpi choursm clempm clipm,lags(1(1)12)
irf create vkitchen,step(36) replace set(irf_final) nose
***Kitchen sink - first shock dated
qui var clstock fclean cffr clwage clcpi choursm clempm clipm,lags(1(1)12)
irf create fkitchen,step(36) replace set(irf_final) nose
***Kitchen sink - scaled by shock size
qui var clstock sclean cffr clwage clcpi choursm clempm clipm,lags(1(1)12)
irf create skitchen,step(36) replace set(irf_final) nose

******DETRENDING
***HP value of 1296 detrending
qui var hflstock clean hfffr hflwage hflcpi hfhoursm hflempm hflipm,lags(1(1)12)
irf create hfkitchen,step(36) replace set(irf_final) nose
***HP linear detrend
qui var ldlstock clean ldffr ldlwage ldlcpi ldhoursm ldlempm ldlipm,lags(1(1)12)
irf create ldkitchen,step(36) replace set(irf_final) nose
***HP standard, no detrending
qui var lstock clean ffr lwage lcpi hoursm lempm lipm,lags(1(1)12)
irf create standard,step(36) replace set(irf_final) nose

