clear
set memory 100m
use irf_final.irf

ren step _step
drop cirf coirf sirf  fevd sf* mse*
drop stdi* stdc* stdf* stds*
*Standardizing on names rather then have loads of variables further below
replace impulse="clean" if impulse=="oclean"|impulse=="sclean"|impulse=="vclean"|impulse=="fclean"|impulse=="xclean"|impulse=="dclean"
replace response="clean" if response=="oclean"|response=="sclean"|response=="vclean"|response=="fclean"|response=="xclean"|response=="dclean"
replace impulse="cvolatbl" if impulse=="volatbl"|impulse=="hfvolatbl"|impulse=="ldvolatbl"|impulse=="c1volatbl"|impulse=="clean"|impulse=="dclean"
replace impulse="cffr" if impulse=="ffr"|impulse=="hfffr"|impulse=="ldffr"|impulse=="c1ffr"|impulse=="ffr"|impulse=="dffr"
replace impulse="clipm" if impulse=="lipm"|impulse=="hflipm"|impulse=="ldlipm"|impulse=="c1lipm"|impulse=="lipm"|impulse=="dlipm"
replace impulse="clempm" if impulse=="lempm"|impulse=="hflempm"|impulse=="ldlempm"|impulse=="c1lempm"|impulse=="lempm"|impulse=="dlempm"
replace impulse="clstock" if impulse=="lstock"|impulse=="hflstock"|impulse=="ldlstock"|impulse=="c1lstock"|impulse=="lstock"|impulse=="dlstock"
replace impulse="chours" if impulse=="hours"|impulse=="hfhours"|impulse=="ldhours"|impulse=="c1hours"|impulse=="hours"|impulse=="dhours"
replace response="clipm" if response=="lipm"|response=="hflipm"|response=="ldlipm"|response=="c1lipm"|response=="lipm"|response=="dlipm"
replace response="clempm" if response=="lempm"|response=="hflempm"|response=="ldlempm"|response=="c1lempm"|response=="lempm"|response=="dlempm"
replace response="cvolatbl" if response=="volatbl"|response=="hfvolatbl"|response=="ldvolatbl"|response=="c1volatbl"|response=="clean"|response=="dclean"
replace response="cffr" if response=="ffr"|response=="c1ffr"|response=="ldffr"|response=="hfffr"|response=="fffr"|response=="dffr"
replace response="clstock" if response=="c1lstock"|response=="ldlstock"|response=="hflstock"|response=="lstock"|response=="dlstock"
replace response="chours" if response=="c1hours"|response=="ldhours"|response=="hfhours"|response=="hours"|response=="dhours"
replace response="clwage" if response=="lwage"
replace response="clcpi" if response=="lcpi"

gen ir=impulse+response
drop impulse response
ren _step step


reshape wide oirf irf stdoirf,i(step irfname) j(ir) string


***THREE STEPS OF NORMALIZATION TO GET MAGNITUDES

****Normalizing the oirfs into the same units as irf
egen cvolratio=max((step==0)*oirfcvolatblcvolatbl),by(irfname)
egen cffrratio=max((step==0)*oirfcffrcffr),by(irfname)
egen clstockratio=max((step==0)*oirfclstockclstock),by(irfname)
replace oirfcvolatblcffr   =oirfcvolatblcffr  /cvolratio
replace oirfcvolatblclwage   =oirfcvolatblclwage  /cvolratio
replace oirfcvolatblclcpi   =oirfcvolatblclcpi  /cvolratio
replace oirfcvolatblclipm  =oirfcvolatblclipm /cvolratio
replace oirfcvolatblclempm =oirfcvolatblclempm/cvolratio
replace oirfcvolatblchours =oirfcvolatblchours/cvolratio
replace oirfcvolatblcvolatbl =oirfcvolatblcvolatbl/cvolratio
replace oirfcffrclipm =oirfcffrclipm /cffrratio
replace oirfcffrclempm=oirfcffrclempm/cffrratio
replace oirfcffrcffr=oirfcffrcffr/cffrratio
replace oirfcffrchours=oirfcffrchours/cffrratio
replace oirfclstockclipm =oirfclstockclipm /clstockratio
replace oirfclstockclempm=oirfclstockclempm/clstockratio
replace stdoirfcvolatblclipm =stdoirfcvolatblclipm /cvolratio
replace stdoirfcvolatblclempm =stdoirfcvolatblclempm /cvolratio
replace stdoirfcvolatblcffr =stdoirfcvolatblcffr /cvolratio
replace stdoirfcvolatblclcpi =stdoirfcvolatblclcpi /cvolratio
replace stdoirfcvolatblclwage =stdoirfcvolatblclwage /cvolratio

****Normalizing the oirfs into meaningful units, vol to a 15 unit shock if actual, stock-market to 5% and interest rates to 1%
gen shock=(irfname=="vbasic"|irfname=="vkitchen"|irfname=="sbasic"|irfname=="skitchen")
gen ratio=15
replace oirfcvolatblclipm =oirfcvolatblclipm *ratio if shock==1
replace oirfcvolatblclempm=oirfcvolatblclempm*ratio if shock==1
replace oirfcvolatblchours=oirfcvolatblchours*ratio if shock==1
replace oirfcvolatblcffr=oirfcvolatblcffr*ratio if shock==1
replace oirfcvolatblclwage=oirfcvolatblclwage*ratio if shock==1
replace oirfclstockclipm =-oirfclstockclipm * 0.05
replace oirfclstockclempm=-oirfclstockclempm* 0.05
replace oirfclstockcffr=oirfclstockcffr* 0.05
replace stdoirfcvolatblclipm=stdoirfcvolatblclipm*ratio  if shock==1
replace stdoirfcvolatblclempm=stdoirfcvolatblclempm*ratio  if shock==1

****Normalizing the impact into % unit for both out and employment
replace oirfcvolatblclipm =oirfcvolatblclipm *100
replace oirfcvolatblclempm=oirfcvolatblclempm*100
*replace oirfcvolatblclhours=oirfcvolatblclhours*100
replace oirfcvolatblclwage=oirfcvolatblclwage*100
replace oirfcvolatblclcpi=oirfcvolatblclcpi*100
replace oirfcffrclipm =oirfcffrclipm *100
replace oirfcffrclempm=oirfcffrclempm*100
replace oirfclstockclipm =oirfclstockclipm *100
replace oirfclstockclempm=oirfclstockclempm*100
replace stdoirfcvolatblclipm =stdoirfcvolatblclipm*100
replace stdoirfcvolatblclempm=stdoirfcvolatblclempm*100
replace stdoirfcvolatblclcpi =stdoirfcvolatblclcpi*100
replace stdoirfcvolatblclwage =stdoirfcvolatblclwage*100

***Generate standard error bands
gen se=1
gen olower=oirfcvolatblclipm-stdoirfcvolatblclipm*se
gen oupper=oirfcvolatblclipm+stdoirfcvolatblclipm*se
gen elower=oirfcvolatblclempm-stdoirfcvolatblclempm*se
gen eupper=oirfcvolatblclempm+stdoirfcvolatblclempm*se
gen flower=oirfcvolatblcffr-stdoirfcvolatblcffr*se
gen fupper=oirfcvolatblcffr+stdoirfcvolatblcffr*se
gen wlower=oirfcvolatblclwage-stdoirfcvolatblclwage*se
gen wupper=oirfcvolatblclwage+stdoirfcvolatblclwage*se
gen plower=oirfcvolatblclcpi-stdoirfcvolatblclcpi*se
gen pupper=oirfcvolatblclcpi+stdoirfcvolatblclcpi*se

lab var step "year"
sa irf,replace

******PLOTTING OUTPUT

*MAIN RESULTS - OUTPUT
scatter oirfcvolatblclipm oirfcffrclipm olower oupper  step if irfname=="kitchen"&step<37,c(l l l l)   lp(l l _ -) s(+ o p p) legend(off) lw(medthick medthick medium medium) lc(black cranberry black black) mc(black cranberry black black)  xlab(0(6)36)

*MAIN RESULTS - EMPLOYMENT
scatter oirfcvolatblclempm oirfcffrclempm elower eupper  step if irfname=="kitchen"&step<37,c(l l l l) lp(l l _ -) s(+ o p p) legend(off) lw(medthick medthick medium medium) lc(black cranberry black black) mc(black cranberry black black)  xlab(0(6)36) 

*INTEREST RATES, PRICES AND WAGES
scatter oirfcvolatblcffr oirfcvolatblclwage oirfcvolatblclcpi  step if irfname=="kitchen"&step<37,c(l l l ) lp(l l l) s(+ d o) legend(off) lw(medthick medthick medthick) lc(black cranberry green) mc(black cranberry green)  xlab(0(6)36)

*********ROBUSTNESS TESTS
*Different shock measures 
*egen z0=sum(oirfcvolatblclipm*(irfname=="kitchen")) , by(step)
egen z1=sum(oirfcvolatblclipm*(irfname=="okitchen")), by(step)
egen z2=sum(oirfcvolatblclipm*(irfname=="skitchen")), by(step)
egen z3=sum(oirfcvolatblclipm*(irfname=="vkitchen")), by(step)
egen z4=sum(oirfcvolatblclipm*(irfname=="fkitchen")), by(step)
scatter z1 z2 z3 z4 step if irfname=="kitchen",c(l l l l l) lp(l l l l) s(+ o s t) legend(off) lc(black cranberry dknavy dkgreen) mc(black cranberry dknavy dkgreen brown)  xlab(0(6)36)

*Variable sets
egen x0=sum(oirfcvolatblclipm*(irfname=="kitchen")) ,by(step)
egen x1=sum(oirfcvolatblclipm*(irfname=="baseline")) ,by(step)
egen x2=sum(oirfcvolatblclipm*(irfname=="bivariate")),by(step)
egen x3=sum(oirfcvolatblclipm*(irfname=="reverse"))  ,by(step)
scatter x0 x1 x2 x3 step if irfname=="kitchen",c(l l l l l) lp(l l l l) s(+ o s t) legend(off) lc(black cranberry dknavy dkgreen) mc(black cranberry dknavy dkgreen brown)  xlab(0(6)36)

*Detrending
egen y1=sum(oirfcvolatblclipm*(irfname=="hfkitchen")),by(step)
egen y2=sum(oirfcvolatblclipm*(irfname=="ldkitchen")),by(step)
egen y4=sum(oirfcvolatblclipm*(irfname=="standard")),by(step)
scatter  y1 y2 y4 step if irfname=="kitchen",c(l l l l l) lp(l l l l) s(+ o s t) legend(off) lc(black cranberry dknavy dkgreen) mc(black cranberry dknavy dkgreen brown)  xlab(0(6)36)

